# -*- coding: utf-8 -*-
"""\
Test the fake fastnumbers module.
"""
from __future__ import unicode_literals

import unicodedata
from math import isnan
from natsort.compat.py23 import PY_VERSION
from natsort.compat.fake_fastnumbers import (
    fast_float,
    fast_int,
)
from hypothesis import (
    given,
)
from hypothesis.strategies import (
    floats,
    integers,
    text,
)

if PY_VERSION >= 3:
    long = int


def is_float(x):
    try:
        float(x)
    except ValueError:
        try:
            unicodedata.numeric(x)
        except (ValueError, TypeError):
            return False
        else:
            return True
    else:
        return True


def not_a_float(x):
    return not is_float(x)


def is_int(x):
    try:
        return x.is_integer()
    except AttributeError:
        try:
            long(x)
        except ValueError:
            try:
                unicodedata.digit(x)
            except (ValueError, TypeError):
                return False
            else:
                return True
        else:
            return True


def not_an_int(x):
    return not is_int(x)


# Each test has an "example" version for demonstrative purposes,
# and a test that uses the hypothesis module.


def test_fast_float_returns_nan_alternate_if_nan_option_is_given():
    assert fast_float('nan', nan=7) == 7


def test_fast_float_converts_float_string_to_float_example():
    assert fast_float('45.8') == 45.8
    assert fast_float('-45') == -45.0
    assert fast_float('45.8e-2', key=len) == 45.8e-2
    assert isnan(fast_float('nan'))
    assert isnan(fast_float('+nan'))
    assert isnan(fast_float('-NaN'))
    assert fast_float('۱۲.۱۲') == 12.12
    assert fast_float('-۱۲.۱۲') == -12.12


@given(floats(allow_nan=False))
def test_fast_float_converts_float_string_to_float(x):
    assert fast_float(repr(x)) == x


def test_fast_float_leaves_string_as_is_example():
    assert fast_float('invalid') == 'invalid'


@given(text().filter(not_a_float).filter(bool))
def test_fast_float_leaves_string_as_is(x):
    assert fast_float(x) == x


def test_fast_float_with_key_applies_to_string_example():
    assert fast_float('invalid', key=len) == len('invalid')


@given(text().filter(not_a_float).filter(bool))
def test_fast_float_with_key_applies_to_string(x):
    assert fast_float(x, key=len) == len(x)


def test_fast_int_leaves_float_string_as_is_example():
    assert fast_int('45.8') == '45.8'
    assert fast_int('nan') == 'nan'
    assert fast_int('inf') == 'inf'


@given(floats().filter(not_an_int))
def test_fast_int_leaves_float_string_as_is(x):
    assert fast_int(repr(x)) == repr(x)


def test_fast_int_converts_int_string_to_int_example():
    assert fast_int('-45') == -45
    assert fast_int('+45') == 45
    assert fast_int('۱۲') == 12
    assert fast_int('-۱۲') == -12


@given(integers())
def test_fast_int_converts_int_string_to_int(x):
    assert fast_int(repr(x)) == x


def test_fast_int_leaves_string_as_is_example():
    assert fast_int('invalid') == 'invalid'


@given(text().filter(not_an_int).filter(bool))
def test_fast_int_leaves_string_as_is(x):
    assert fast_int(x) == x


def test_fast_int_with_key_applies_to_string_example():
    assert fast_int('invalid', key=len) == len('invalid')


@given(text().filter(not_an_int).filter(bool))
def test_fast_int_with_key_applies_to_string(x):
    assert fast_int(x, key=len) == len(x)



# -*- coding: utf-8 -*-
"""These test the utils.py functions."""
from __future__ import unicode_literals

from pytest import raises
from natsort.ns_enum import ns
from natsort.utils import (
    _float_sign_exp_re,
    _float_nosign_exp_re,
    _float_sign_noexp_re,
    _float_nosign_noexp_re,
    _int_nosign_re,
    _int_sign_re,
    _parse_string_factory,
    _parse_path_factory,
)
from natsort.compat.py23 import py23_str, PY_VERSION
from natsort.compat.fastnumbers import (
    fast_float,
    fast_int,
)
from slow_splitters import (
    int_splitter,
    float_splitter,
)
from hypothesis import (
    given,
    example,
)
from hypothesis.strategies import (
    lists,
    text,
    floats,
    integers,
)

if PY_VERSION >= 3:
    long = int


def whitespace_check(x):
    """Simplifies testing"""
    try:
        if x.isspace():
            return x in ' \t\n\r\f\v'
        else:
            return True
    except (AttributeError, TypeError):
        return True


def no_op(x):
    """A function that does nothing."""
    return x


def tuple2(x, dummy):
    """Make the input a tuple."""
    return tuple(x)


# Each test has an "example" version for demonstrative purposes,
# and a test that uses the hypothesis module.


def test_parse_string_factory_raises_TypeError_if_given_a_number_example():
    with raises(TypeError):
        assert _parse_string_factory(0, '', _float_sign_exp_re.split, no_op, fast_float, tuple2)(50.0)


@given(floats())
def test_parse_string_factory_raises_TypeError_if_given_a_number(x):
    with raises(TypeError):
        assert _parse_string_factory(0, '', _float_sign_exp_re.split, no_op, fast_float, tuple2)(x)


def test_parse_string_factory_only_parses_digits_with_nosign_int_example():
    assert _parse_string_factory(0, '', _int_nosign_re.split, no_op, fast_int, tuple2)('a5+5.034e-1') == ('a', 5, '+', 5, '.', 34, 'e-', 1)


@given(lists(elements=floats() | text().filter(whitespace_check) | integers(), min_size=1, max_size=10))
@example([10000000000000000000000000000000000000000000000000000000000000000000000000,
          100000000000000000000000000000000000000000000000000000000000000000000000000,
          100000000000000000000000000000000000000000000000000000000000000000000000000])
def test_parse_string_factory_only_parses_digits_with_nosign_int(x):
    s = ''.join(repr(y) if type(y) in (float, long, int) else y for y in x)
    assert _parse_string_factory(0, '', _int_nosign_re.split, no_op, fast_int, tuple2)(s) == int_splitter(s, False, '')


def test_parse_string_factory_parses_digit_with_sign_with_signed_int_example():
    assert _parse_string_factory(0, '', _int_sign_re.split, no_op, fast_int, tuple2)('a5+5.034e-1') == ('a', 5, '', 5, '.', 34, 'e', -1)


@given(lists(elements=floats() | text().filter(whitespace_check) | integers(), min_size=1, max_size=10))
def test_parse_string_factory_parses_digit_with_sign_with_signed_int(x):
    s = ''.join(repr(y) if type(y) in (float, long, int) else y for y in x)
    assert _parse_string_factory(0, '', _int_sign_re.split, no_op, fast_int, tuple2)(s) == int_splitter(s, True, '')


def test_parse_string_factory_only_parses_float_with_nosign_noexp_float_example():
    assert _parse_string_factory(0, '', _float_nosign_noexp_re.split, no_op, fast_float, tuple2)('a5+5.034e-1') == ('a', 5.0, '+', 5.034, 'e-', 1.0)


@given(lists(elements=floats(allow_nan=False) | text().filter(whitespace_check) | integers(), min_size=1, max_size=10))
def test_parse_string_factory_only_parses_float_with_nosign_noexp_float(x):
    s = ''.join(repr(y) if type(y) in (float, long, int) else y for y in x)
    assert _parse_string_factory(0, '', _float_nosign_noexp_re.split, no_op, fast_float, tuple2)(s) == float_splitter(s, False, False, '')


def test_parse_string_factory_only_parses_float_with_exponent_with_nosign_exp_float_example():
    assert _parse_string_factory(0, '', _float_nosign_exp_re.split, no_op, fast_float, tuple2)('a5+5.034e-1') == ('a', 5.0, '+', 0.5034)


@given(lists(elements=floats(allow_nan=False) | text().filter(whitespace_check) | integers(), min_size=1, max_size=10))
def test_parse_string_factory_only_parses_float_with_exponent_with_nosign_exp_float(x):
    s = ''.join(repr(y) if type(y) in (float, long, int) else y for y in x)
    assert _parse_string_factory(0, '', _float_nosign_exp_re.split, no_op, fast_float, tuple2)(s) == float_splitter(s, False, True, '')


def test_parse_string_factory_only_parses_float_with_sign_with_sign_noexp_float_example():
    assert _parse_string_factory(0, '', _float_sign_noexp_re.split, no_op, fast_float, tuple2)('a5+5.034e-1') == ('a', 5.0, '', 5.034, 'e', -1.0)


@given(lists(elements=floats(allow_nan=False) | text().filter(whitespace_check) | integers(), min_size=1, max_size=10))
def test_parse_string_factory_only_parses_float_with_sign_with_sign_noexp_float(x):
    s = ''.join(repr(y) if type(y) in (float, long, int) else y for y in x)
    assert _parse_string_factory(0, '', _float_sign_noexp_re.split, no_op, fast_float, tuple2)(s) == float_splitter(s, True, False, '')


def test_parse_string_factory_parses_float_with_sign_exp_float_example():
    assert _parse_string_factory(0, '', _float_sign_exp_re.split, no_op, fast_float, tuple2)('a5+5.034e-1') == ('a', 5.0, '', 0.5034)
    assert _parse_string_factory(0, '', _float_sign_exp_re.split, no_op, fast_float, tuple2)('6a5+5.034e-1') == ('', 6.0, 'a', 5.0, '', 0.5034)


@given(lists(elements=floats(allow_nan=False) | text().filter(whitespace_check) | integers(), min_size=1, max_size=10))
def test_parse_string_factory_parses_float_with_sign_exp_float(x):
    s = ''.join(repr(y) if type(y) in (float, long, int) else y for y in x)
    assert _parse_string_factory(0, '', _float_sign_exp_re.split, no_op, fast_float, tuple2)(s) == float_splitter(s, True, True, '')


def test_parse_string_factory_selects_pre_function_value_if_not_dumb():
    def tuple2(x, orig):
        """Make the input a tuple."""
        return (orig[0], tuple(x))
    assert _parse_string_factory(0, '', _int_nosign_re.split, py23_str.upper, fast_float, tuple2)('a5+5.034e-1') == ('A', ('A', 5, '+', 5, '.', 34, 'E-', 1))
    assert _parse_string_factory(ns._DUMB, '', _int_nosign_re.split, py23_str.upper, fast_float, tuple2)('a5+5.034e-1') == ('A', ('A', 5, '+', 5, '.', 34, 'E-', 1))
    assert _parse_string_factory(ns.LOCALE, '', _int_nosign_re.split, py23_str.upper, fast_float, tuple2)('a5+5.034e-1') == ('A', ('A', 5, '+', 5, '.', 34, 'E-', 1))
    assert _parse_string_factory(ns.LOCALE | ns._DUMB, '', _int_nosign_re.split, py23_str.upper, fast_float, tuple2)('a5+5.034e-1') == ('a', ('A', 5, '+', 5, '.', 34, 'E-', 1))


def test_parse_path_function_parses_string_as_path_then_as_string():
    splt = _parse_string_factory(0, '', _float_sign_exp_re.split, no_op, fast_float, tuple2)
    assert _parse_path_factory(splt)('/p/Folder (10)/file34.5nm (2).tar.gz') == (('/',), ('p',), ('Folder (', 10.0, ')',), ('file', 34.5, 'nm (', 2.0, ')'), ('.tar',), ('.gz',))
    assert _parse_path_factory(splt)('../Folder (10)/file (2).tar.gz') == (('..',), ('Folder (', 10.0, ')',), ('file (', 2.0, ')'), ('.tar',), ('.gz',))
    assert _parse_path_factory(splt)('Folder (10)/file.f34.5nm (2).tar.gz') == (('Folder (', 10.0, ')',), ('file.f', 34.5, 'nm (', 2.0, ')'), ('.tar',), ('.gz',))



# -*- coding: utf-8 -*-
"""\
Here are a collection of examples of how this module can be used.
See the README or the natsort homepage for more details.
"""
from __future__ import unicode_literals, print_function
from operator import itemgetter
from natsort.compat.py23 import PY_VERSION
from natsort import (
    natsorted,
    index_natsorted,
    versorted,
    index_versorted,
    humansorted,
    index_humansorted,
    realsorted,
    index_realsorted,
    order_by_index,
    ns,
    decoder,
    as_ascii,
    as_utf8,
)


def test_decoder_returns_function_that_can_decode_bytes_but_return_non_bytes_as_is():
    f = decoder('latin1')
    a = 'bytes'
    b = 14
    assert f(b'bytes') == a
    assert f(b) is b  # returns as-is, same object ID
    if PY_VERSION >= 3:
        assert f(a) is a  # same object returned on Python3 b/c only bytes has decode
    else:
        assert f(a) is not a
        assert f(a) == a  # not same object on Python2 because str can decode


def test_as_ascii_returns_bytes_as_ascii():
    assert decoder('ascii')(b'bytes') == as_ascii(b'bytes')


def test_as_utf8_returns_bytes_as_utf8():
    assert decoder('utf8')(b'bytes') == as_utf8(b'bytes')


def test_versorted_returns_results_identical_to_natsorted():
    a = ['1.9.9a', '1.11', '1.9.9b', '1.11.4', '1.10.1']
    # versorted is retained for backwards compatibility
    assert versorted(a) == natsorted(a)


def test_realsorted_returns_results_identical_to_natsorted_with_REAL():
    a = ['a50', 'a51.', 'a50.31', 'a-50', 'a50.4', 'a5.034e1', 'a50.300']
    assert realsorted(a) == natsorted(a, alg=ns.REAL)


def test_humansorted_returns_results_identical_to_natsorted_with_LOCALE():
    a = ['Apple', 'corn', 'Corn', 'Banana', 'apple', 'banana']
    assert humansorted(a) == natsorted(a, alg=ns.LOCALE)


def test_index_natsorted_returns_integer_list_of_sort_order_for_input_list():
    a = ['num3', 'num5', 'num2']
    b = ['foo', 'bar', 'baz']
    index = index_natsorted(a)
    assert index == [2, 0, 1]
    assert [a[i] for i in index] == ['num2', 'num3', 'num5']
    assert [b[i] for i in index] == ['baz', 'foo', 'bar']


def test_index_natsorted_returns_reversed_integer_list_of_sort_order_for_input_list_with_reverse_option():
    a = ['num3', 'num5', 'num2']
    assert index_natsorted(a, reverse=True) == [1, 0, 2]


def test_index_natsorted_applies_key_function_before_sorting():
    c = [('a', 'num3'), ('b', 'num5'), ('c', 'num2')]
    assert index_natsorted(c, key=itemgetter(1)) == [2, 0, 1]


def test_index_natsorted_handles_unorderable_types_error_on_Python3():
    a = [46, '5a5b2', 'af5', '5a5-4']
    assert index_natsorted(a) == [3, 1, 0, 2]


def test_index_natsorted_returns_integer_list_of_nested_input_list():
    data = [['a1', 'a5'], ['a1', 'a40'], ['a10', 'a1'], ['a2', 'a5']]
    assert index_natsorted(data) == [0, 1, 3, 2]


def test_index_natsorted_returns_integer_list_in_proper_order_for_input_paths_with_PATH():
    a = ['/p/Folder (10)/',
         '/p/Folder/',
         '/p/Folder (1)/']
    assert index_natsorted(a, alg=ns.PATH) == [1, 2, 0]


def test_index_versorted_returns_results_identical_to_index_natsorted():
    a = ['1.9.9a', '1.11', '1.9.9b', '1.11.4', '1.10.1']
    # index_versorted is retained for backwards compatibility
    assert index_versorted(a) == index_natsorted(a)


def test_index_realsorted_returns_results_identical_to_index_natsorted_with_REAL():
    a = ['a50', 'a51.', 'a50.31', 'a-50', 'a50.4', 'a5.034e1', 'a50.300']
    assert index_realsorted(a) == index_natsorted(a, alg=ns.REAL)


def test_index_humansorted_returns_results_identical_to_index_natsorted_with_LOCALE():
    a = ['Apple', 'corn', 'Corn', 'Banana', 'apple', 'banana']
    assert index_humansorted(a) == index_natsorted(a, alg=ns.LOCALE)


def test_order_by_index_sorts_list_according_to_order_of_integer_list():
    a = ['num3', 'num5', 'num2']
    index = [2, 0, 1]
    assert order_by_index(a, index) == ['num2', 'num3', 'num5']
    assert order_by_index(a, index) == [a[i] for i in index]


def test_order_by_index_returns_generator_with_iter_True():
    a = ['num3', 'num5', 'num2']
    index = [2, 0, 1]
    assert order_by_index(a, index, True) != [a[i] for i in index]
    assert list(order_by_index(a, index, True)) == [a[i] for i in index]



# -*- coding: utf-8 -*-
"""These test the utils.py functions."""
from __future__ import unicode_literals

import pytest
import locale
from operator import methodcaller
from natsort.ns_enum import ns
from natsort.utils import _input_string_transform_factory
from natsort.compat.py23 import NEWPY
from compat.locale import (
    load_locale,
    has_locale_de_DE,
)
from hypothesis import (
    given,
)
from hypothesis.strategies import (
    text,
    integers,
    lists,
)


# Each test has an "example" version for demonstrative purposes,
# and a test that uses the hypothesis module.


def test_input_string_transform_factory_is_no_op_for_no_alg_options_examples():
    x = 'feijGGAd'
    assert _input_string_transform_factory(0)(x) is x


@given(text())
def test_input_string_transform_factory_is_no_op_for_no_alg_options(x):
    assert _input_string_transform_factory(0)(x) is x


def test_input_string_transform_factory_performs_casefold_with_IGNORECASE_examples():
    x = 'feijGGAd'
    if NEWPY:
        assert _input_string_transform_factory(ns.IGNORECASE)(x) == x.casefold()
    else:
        assert _input_string_transform_factory(ns.IGNORECASE)(x) == x.lower()


@given(text())
def test_input_string_transform_factory_performs_casefold_with_IGNORECASE(x):
    if NEWPY:
        assert _input_string_transform_factory(ns.IGNORECASE)(x) == x.casefold()
    else:
        assert _input_string_transform_factory(ns.IGNORECASE)(x) == x.lower()


def test_input_string_transform_factory_performs_swapcase_with_DUMB_examples():
    x = 'feijGGAd'
    assert _input_string_transform_factory(ns._DUMB)(x) == x.swapcase()


@given(text())
def test_input_string_transform_factory_performs_swapcase_with_DUMB(x):
    assert _input_string_transform_factory(ns._DUMB)(x) == x.swapcase()


def test_input_string_transform_factory_performs_swapcase_with_LOWERCASEFIRST_example():
    x = 'feijGGAd'
    assert _input_string_transform_factory(ns.LOWERCASEFIRST)(x) == x.swapcase()


@given(text())
def test_input_string_transform_factory_performs_swapcase_with_LOWERCASEFIRST(x):
    x = 'feijGGAd'
    assert _input_string_transform_factory(ns.LOWERCASEFIRST)(x) == x.swapcase()


def test_input_string_transform_factory_is_no_op_with_both_LOWERCASEFIRST_AND_DUMB_example():
    x = 'feijGGAd'
    assert _input_string_transform_factory(ns._DUMB | ns.LOWERCASEFIRST)(x) is x


@given(text())
def test_input_string_transform_factory_is_no_op_with_both_LOWERCASEFIRST_AND_DUMB(x):
    assert _input_string_transform_factory(ns._DUMB | ns.LOWERCASEFIRST)(x) is x


def test_input_string_transform_factory_performs_swapcase_and_casefold_both_LOWERCASEFIRST_AND_IGNORECASE_example():
    x = 'feijGGAd'
    if NEWPY:
        assert _input_string_transform_factory(ns.IGNORECASE | ns.LOWERCASEFIRST)(x) == x.swapcase().casefold()
    else:
        assert _input_string_transform_factory(ns.IGNORECASE | ns.LOWERCASEFIRST)(x) == x.swapcase().lower()


@given(text())
def test_input_string_transform_factory_performs_swapcase_and_casefold_both_LOWERCASEFIRST_AND_IGNORECASE(x):
    if NEWPY:
        assert _input_string_transform_factory(ns.IGNORECASE | ns.LOWERCASEFIRST)(x) == x.swapcase().casefold()
    else:
        assert _input_string_transform_factory(ns.IGNORECASE | ns.LOWERCASEFIRST)(x) == x.swapcase().lower()


def test_input_string_transform_factory_removes_thousands_separator_with_LOCALE_example():
    load_locale('en_US')
    x = '12,543,642,642.534,534,980'  # Without FLOAT it does not account for decimal.
    assert _input_string_transform_factory(ns.LOCALE)(x) == '12543642642.534534980'
    x = '12,543,642,642.534,534,980'  # LOCALEALPHA doesn't do anything... need LOCALENUM
    assert _input_string_transform_factory(ns.LOCALEALPHA)(x) == '12,543,642,642.534,534,980'
    locale.setlocale(locale.LC_ALL, str(''))


@given(lists(elements=integers(), min_size=4, max_size=20))
def test_input_string_transform_factory_removes_thousands_separator_with_LOCALE(x):
    load_locale('en_US')
    t = ''.join(map(methodcaller('rstrip', 'lL'), map(str, map(abs, x))))  # Remove negative signs trailing L
    s = ''
    for i, y in enumerate(reversed(t), 1):
        s = y + s
        if i % 3 == 0 and i != len(t):
            s = ',' + s
    assert _input_string_transform_factory(ns.LOCALE)(s) == t
    locale.setlocale(locale.LC_ALL, str(''))


def test_input_string_transform_factory_removes_thousands_separator_and_is_float_aware_with_LOCALE_and_FLOAT_example():
    x = '12,543,642,642.534,534,980'
    assert _input_string_transform_factory(ns.LOCALE | ns.FLOAT)(x) == '12543642642.534,534980'


@given(lists(elements=integers(), min_size=4, max_size=20), lists(elements=integers(), min_size=4, max_size=20))
def test_input_string_transform_factory_removes_thousands_separator_and_is_float_aware_with_LOCALE_and_FLOAT(x, y):
    load_locale('en_US')
    t = ''.join(map(methodcaller('rstrip', 'lL'), map(str, map(abs, x))))  # Remove negative signs trailing L
    s = ''
    for i, z in enumerate(reversed(t), 1):
        s = z + s
        if i % 3 == 0 and i != len(t):
            s = ',' + s
    u = ''.join(map(methodcaller('rstrip', 'lL'), map(str, map(abs, y))))  # Remove negative signs trailing L
    v = ''
    for i, z in enumerate(reversed(u), 1):
        v = z + v
        if i % 3 == 0 and i != len(u):
            v = ',' + v
    # Remove all but first comma.
    a = v.split(',', 1)
    p = a[0] + ',' + a[1].replace(',', '')
    assert _input_string_transform_factory(ns.LOCALE)('.'.join([s, v])) == '.'.join([t, u])
    assert _input_string_transform_factory(ns.LOCALE | ns.FLOAT)('.'.join([s, v])) == '.'.join([t, p])
    locale.setlocale(locale.LC_ALL, str(''))


# These might be too much to test with hypothesis.


def test_input_string_transform_factory_leaves_invalid_thousands_separator_with_LOCALE_example():
    load_locale('en_US')
    x = '12,543,642642.5345,34980'
    assert _input_string_transform_factory(ns.LOCALE)(x) == '12543,642642.5345,34980'
    x = '12,59443,642,642.53,4534980'
    assert _input_string_transform_factory(ns.LOCALE)(x) == '12,59443,642642.53,4534980'
    x = '12543,642,642.5,34534980'
    assert _input_string_transform_factory(ns.LOCALE)(x) == '12543,642642.5,34534980'
    locale.setlocale(locale.LC_ALL, str(''))


# @pytest.mark.skipif(not has_locale_de_DE or dumb_sort(), reason='requires de_DE locale and working locale')
@pytest.mark.skipif(not has_locale_de_DE, reason='requires de_DE locale and working locale')
def test_input_string_transform_factory_replaces_decimal_separator_with_LOCALE_example():
    load_locale('de_DE')
    x = '1543,753'
    assert _input_string_transform_factory(ns.LOCALE)(x) == '1543,753'  # Does nothing without FLOAT
    assert _input_string_transform_factory(ns.LOCALE | ns.FLOAT)(x) == '1543.753'
    assert _input_string_transform_factory(ns.LOCALEALPHA)(x) == '1543,753'  # LOCALEALPHA doesn't do anything... need LOCALENUM
    locale.setlocale(locale.LC_ALL, str(''))


# @pytest.mark.skipif(not has_locale_de_DE or dumb_sort(), reason='requires de_DE locale and working locale')
@pytest.mark.skipif(not has_locale_de_DE, reason='requires de_DE locale and working locale')
def test_input_string_transform_factory_does_not_replace_invalid_decimal_separator_with_LOCALE_example():
    load_locale('de_DE')
    x = '154s,t53'
    assert _input_string_transform_factory(ns.LOCALE | ns.FLOAT)(x) == '154s,t53'
    locale.setlocale(locale.LC_ALL, str(''))



# -*- coding: utf-8 -*-
"""\
This file contains functions to profile natsorted with different
inputs and different settings.
"""
from __future__ import print_function
import cProfile
import sys

sys.path.insert(0, '.')
from natsort import natsort_keygen, ns
from natsort.compat.py23 import py23_range
import locale
locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')

# Samples to parse
number = 14695498
int_string = '43493'
float_string = '-434.93e7'
plain_string = 'hello world'
fancy_string = '7abba9342fdab'
a_path = '/p/Folder (1)/file (1).tar.gz'
some_bytes = b'these are bytes'
a_list = ['hello', 'goodbye', '74']

basic_key = natsort_keygen()
real_key = natsort_keygen(alg=ns.REAL)
path_key = natsort_keygen(alg=ns.PATH)
locale_key = natsort_keygen(alg=ns.LOCALE)


def prof_time_to_generate():
    print('*** Generate Plain Key ***')
    for _ in py23_range(100000):
        natsort_keygen()
cProfile.run('prof_time_to_generate()', sort='time')


def prof_parsing(a, msg, key=basic_key):
    print(msg)
    for _ in py23_range(100000):
        key(a)
cProfile.run('prof_parsing(int_string, "*** Basic Call, Int as String ***")', sort='time')
cProfile.run('prof_parsing(float_string, "*** Basic Call, Float as String ***")', sort='time')
cProfile.run('prof_parsing(float_string, "*** Real Call ***", real_key)', sort='time')
cProfile.run('prof_parsing(number, "*** Basic Call, Number ***")', sort='time')
cProfile.run('prof_parsing(fancy_string, "*** Basic Call, Mixed String ***")', sort='time')
cProfile.run('prof_parsing(some_bytes, "*** Basic Call, Byte String ***")', sort='time')
cProfile.run('prof_parsing(a_path, "*** Path Call ***", path_key)', sort='time')
cProfile.run('prof_parsing(a_list, "*** Basic Call, Recursive ***")', sort='time')
cProfile.run('prof_parsing("434,930,000 dollars", "*** Locale Call ***", locale_key)', sort='time')



# -*- coding: utf-8 -*-
from __future__ import (
    print_function,
    division,
    unicode_literals,
    absolute_import
)

# Std. lib imports.
import locale

# Local imports
from natsort.compat.py23 import py23_str, py23_unichr, py23_range


def load_locale(x):
    """ Convenience to load a locale, trying ISO8859-1 first."""
    try:
        locale.setlocale(locale.LC_ALL, str('{0}.ISO8859-1'.format(x)))
    except locale.Error:
        locale.setlocale(locale.LC_ALL, str('{0}.UTF-8'.format(x)))

# Check if de_DE is installed.
try:
    load_locale('de_DE')
    has_locale_de_DE = True
except locale.Error:
    has_locale_de_DE = False

# Depending on the python version, use lower or casefold
# to make a string lowercase.
try:
    low = py23_str.casefold
except AttributeError:
    low = py23_str.lower

# There are some unicode values that are known failures on BSD systems
# that has nothing to do with natsort (a ValueError is raised by strxfrm).
# Let's filter them out.
try:
    bad_uni_chars = set(py23_unichr(x) for x in py23_range(0X10fefd,
                                                           0X10ffff+1))
except ValueError:
    # Narrow unicode build... no worries.
    bad_uni_chars = set()



# -*- coding: utf-8 -*-
from __future__ import (
    print_function,
    division,
    unicode_literals,
    absolute_import
)

from distutils.version import StrictVersion

# If the user has fastnumbers installed, they will get great speed
# benefits. If not, we use the simulated functions that come with natsort.
try:
    from fastnumbers import (
        fast_float,
        fast_int,
    )
    import fastnumbers
    # Require >= version 0.7.1.
    if StrictVersion(fastnumbers.__version__) < StrictVersion('0.7.1'):
        raise ImportError  # pragma: no cover
except ImportError:
    from natsort.compat.fake_fastnumbers import (
        fast_float,
        fast_int,
    )



# -*- coding: utf-8 -*-
"""These test the utils.py functions."""
from __future__ import unicode_literals

import pathlib
import string
from itertools import chain
from operator import neg as op_neg
from pytest import raises
from natsort.ns_enum import ns
from natsort.utils import (
    _sep_inserter,
    _args_to_enum,
    _regex_chooser,
    _float_sign_exp_re,
    _float_nosign_exp_re,
    _float_sign_noexp_re,
    _float_nosign_noexp_re,
    _int_nosign_re,
    _int_sign_re,
    _do_decoding,
    _path_splitter,
    _groupletters,
    chain_functions,
)
from natsort.compat.py23 import py23_str, py23_cmp
from natsort.compat.locale import null_string_locale
from slow_splitters import (
    sep_inserter,
    add_leading_space_if_first_is_num,
)
from compat.locale import low
from hypothesis import (
    given,
)
from hypothesis.strategies import (
    sampled_from,
    lists,
    text,
    integers,
)


def test_do_decoding_decodes_bytes_string_to_unicode():
    assert type(_do_decoding(b'bytes', 'ascii')) is py23_str
    assert _do_decoding(b'bytes', 'ascii') == 'bytes'
    assert _do_decoding(b'bytes', 'ascii') == b'bytes'.decode('ascii')


def test_args_to_enum_raises_TypeError_for_invalid_argument():
    with raises(TypeError):
        _args_to_enum(**{'alf': 0})


def test_args_to_enum_converts_signed_exp_float_to_ns_F():
    # number_type, signed, exp, as_path, py3_safe
    assert _args_to_enum(**{'number_type': float,
                            'signed': True,
                            'exp': True}) == ns.F | ns.S


def test_args_to_enum_converts_signed_noexp_float_to_ns_FN():
    # number_type, signed, exp, as_path, py3_safe
    assert _args_to_enum(**{'number_type': float,
                            'signed': True,
                            'exp': False}) == ns.F | ns.N | ns.S


def test_args_to_enum_converts_unsigned_exp_float_to_ns_FU():
    # number_type, signed, exp, as_path, py3_safe
    assert _args_to_enum(**{'number_type': float,
                            'signed': False,
                            'exp': True}) == ns.F | ns.U
    # unsigned is default
    assert _args_to_enum(**{'number_type': float,
                            'signed': False,
                            'exp': True}) == ns.F


def test_args_to_enum_converts_unsigned_unexp_float_to_ns_FNU():
    # number_type, signed, exp, as_path, py3_safe
    assert _args_to_enum(**{'number_type': float,
                            'signed': False,
                            'exp': False}) == ns.F | ns.U | ns.N


def test_args_to_enum_converts_float_and_path_and_py3safe_to_ns_FPT():
    # number_type, signed, exp, as_path, py3_safe
    assert _args_to_enum(**{'number_type': float,
                            'as_path': True,
                            'py3_safe': True}) == ns.F | ns.P | ns.T


def test_args_to_enum_converts_int_and_path_to_ns_IP():
    # number_type, signed, exp, as_path, py3_safe
    assert _args_to_enum(**{'number_type': int, 'as_path': True}) == ns.I | ns.P


def test_args_to_enum_converts_unsigned_int_and_py3safe_to_ns_IUT():
    # number_type, signed, exp, as_path, py3_safe
    assert _args_to_enum(**{'number_type': int,
                            'signed': False,
                            'py3_safe': True}) == ns.I | ns.U | ns.T


def test_args_to_enum_converts_None_to_ns_IU():
    # number_type, signed, exp, as_path, py3_safe
    assert _args_to_enum(**{'number_type': None,
                            'exp': True}) == ns.I | ns.U


def test_regex_chooser_returns_correct_regular_expression_object():
    assert _regex_chooser[ns.INT] is _int_nosign_re
    assert _regex_chooser[ns.INT | ns.NOEXP] is _int_nosign_re
    assert _regex_chooser[ns.INT | ns.SIGNED] is _int_sign_re
    assert _regex_chooser[ns.INT | ns.SIGNED | ns.NOEXP] is _int_sign_re
    assert _regex_chooser[ns.FLOAT] is _float_nosign_exp_re
    assert _regex_chooser[ns.FLOAT | ns.NOEXP] is _float_nosign_noexp_re
    assert _regex_chooser[ns.FLOAT | ns.SIGNED] is _float_sign_exp_re
    assert _regex_chooser[ns.FLOAT | ns.SIGNED | ns.NOEXP] is _float_sign_noexp_re


def test_ns_enum_values_have_are_as_expected():
    # Defaults
    assert ns.TYPESAFE == 0
    assert ns.INT == 0
    assert ns.VERSION == 0
    assert ns.DIGIT == 0
    assert ns.UNSIGNED == 0

    # Aliases
    assert ns.TYPESAFE == ns.T
    assert ns.INT == ns.I
    assert ns.VERSION == ns.V
    assert ns.DIGIT == ns.D
    assert ns.UNSIGNED == ns.U
    assert ns.FLOAT == ns.F
    assert ns.SIGNED == ns.S
    assert ns.NOEXP == ns.N
    assert ns.PATH == ns.P
    assert ns.LOCALEALPHA == ns.LA
    assert ns.LOCALENUM == ns.LN
    assert ns.LOCALE == ns.L
    assert ns.IGNORECASE == ns.IC
    assert ns.LOWERCASEFIRST == ns.LF
    assert ns.GROUPLETTERS == ns.G
    assert ns.UNGROUPLETTERS == ns.UG
    assert ns.CAPITALFIRST == ns.C
    assert ns.UNGROUPLETTERS == ns.CAPITALFIRST
    assert ns.NANLAST == ns.NL
    assert ns.COMPATIBILITYNORMALIZE == ns.CN
    assert ns.NUMAFTER == ns.NA

    # Convenience
    assert ns.LOCALE == ns.LOCALEALPHA | ns.LOCALENUM
    assert ns.REAL == ns.FLOAT | ns.SIGNED
    assert ns._NUMERIC_ONLY == ns.REAL | ns.NOEXP


def test_chain_functions_is_a_no_op_if_no_functions_are_given():
    x = 2345
    assert chain_functions([])(x) is x


def test_chain_functions_does_one_function_if_one_function_is_given():
    x = '2345'
    assert chain_functions([len])(x) == 4


def test_chain_functions_combines_functions_in_given_order():
    x = 2345
    assert chain_functions([str, len, op_neg])(x) == -len(str(x))


# Each test has an "example" version for demonstrative purposes,
# and a test that uses the hypothesis module.

def test_groupletters_returns_letters_with_lowercase_transform_of_letter_example():
    assert _groupletters('HELLO') == 'hHeElLlLoO'
    assert _groupletters('hello') == 'hheelllloo'


@given(text().filter(bool))
def test_groupeletters_returns_letters_with_lowercase_transform_of_letter(x):
    assert _groupletters(x) == ''.join(chain.from_iterable([low(y), y] for y in x))


def test_sep_inserter_does_nothing_if_no_numbers_example():
    assert list(_sep_inserter(iter(['a', 'b', 'c']), '')) == ['a', 'b', 'c']
    assert list(_sep_inserter(iter(['a']), '')) == ['a']


def test_sep_inserter_does_nothing_if_only_one_number_example():
    assert list(_sep_inserter(iter(['a', 5]), '')) == ['a', 5]


def test_sep_inserter_inserts_separator_string_between_two_numbers_example():
    assert list(_sep_inserter(iter([5, 9]), '')) == ['', 5, '', 9]
    assert list(_sep_inserter(iter([5, 9]), null_string_locale)) == [null_string_locale, 5, null_string_locale, 9]


@given(lists(elements=text().filter(bool) | integers()))
def test_sep_inserter_inserts_separator_between_two_numbers(x):
    assert list(_sep_inserter(iter(x), '')) == list(add_leading_space_if_first_is_num(sep_inserter(x, ''), ''))


def test_path_splitter_splits_path_string_by_separator_example():
    z = '/this/is/a/path'
    assert tuple(_path_splitter(z)) == tuple(pathlib.Path(z).parts)
    z = pathlib.Path('/this/is/a/path')
    assert tuple(_path_splitter(z)) == tuple(pathlib.Path(z).parts)


@given(lists(sampled_from(string.ascii_letters), min_size=2).filter(all))
def test_path_splitter_splits_path_string_by_separator(x):
    z = py23_str(pathlib.Path(*x))
    assert tuple(_path_splitter(z)) == tuple(pathlib.Path(z).parts)


def test_path_splitter_splits_path_string_by_separator_and_removes_extension_example():
    z = '/this/is/a/path/file.exe'
    y = tuple(pathlib.Path(z).parts)
    assert tuple(_path_splitter(z)) == y[:-1] + (pathlib.Path(z).stem, pathlib.Path(z).suffix)


@given(lists(sampled_from(string.ascii_letters), min_size=3).filter(all))
def test_path_splitter_splits_path_string_by_separator_and_removes_extension(x):
    z = py23_str(pathlib.Path(*x[:-2])) + '.' + x[-1]
    y = tuple(pathlib.Path(z).parts)
    assert tuple(_path_splitter(z)) == y[:-1] + (pathlib.Path(z).stem, pathlib.Path(z).suffix)


@given(integers())
def test_py23_cmp(x):
    assert py23_cmp(x, x) == 0
    assert py23_cmp(x, x + 1) < 0
    assert py23_cmp(x, x - 1) > 0



# -*- coding: utf-8 -*-
"""These test the utils.py functions."""
from __future__ import unicode_literals

from natsort.ns_enum import ns
from natsort.utils import _parse_number_factory
from hypothesis import (
    given,
)
from hypothesis.strategies import (
    floats,
    integers,
)


# Each test has an "example" version for demonstrative purposes,
# and a test that uses the hypothesis module.


def test_parse_number_factory_makes_function_that_returns_tuple_example():
    assert _parse_number_factory(0, '', '')(57) == ('', 57)
    assert _parse_number_factory(0, '', '')(float('nan')) == ('', float('-inf'))
    assert _parse_number_factory(ns.NANLAST, '', '')(float('nan')) == ('', float('+inf'))


@given(floats(allow_nan=False) | integers())
def test_parse_number_factory_makes_function_that_returns_tuple(x):
    assert _parse_number_factory(0, '', '')(x) == ('', x)


def test_parse_number_factory_with_PATH_makes_function_that_returns_nested_tuple_example():
    assert _parse_number_factory(ns.PATH, '', '')(57) == (('', 57),)


@given(floats(allow_nan=False) | integers())
def test_parse_number_factory_with_PATH_makes_function_that_returns_nested_tuple(x):
    assert _parse_number_factory(ns.PATH, '', '')(x) == (('', x),)


def test_parse_number_factory_with_UNGROUPLETTERS_LOCALE_makes_function_that_returns_nested_tuple_example():
    assert _parse_number_factory(ns.UNGROUPLETTERS | ns.LOCALE, '', 'xx')(57) == (('xx',), ('', 57))


@given(floats(allow_nan=False) | integers())
def test_parse_number_factory_with_UNGROUPLETTERS_LOCALE_makes_function_that_returns_nested_tuple(x):
    assert _parse_number_factory(ns.UNGROUPLETTERS | ns.LOCALE, '', 'xx')(x) == (('xx',), ('', x))


def test_parse_number_factory_with_PATH_UNGROUPLETTERS_LOCALE_makes_function_that_returns_nested_tuple_example():
    assert _parse_number_factory(ns.PATH | ns.UNGROUPLETTERS | ns.LOCALE, '', 'xx')(57) == ((('xx',), ('', 57)),)


@given(floats(allow_nan=False) | integers())
def test_parse_number_factory_with_PATH_UNGROUPLETTERS_LOCALE_makes_function_that_returns_nested_tuple(x):
    assert _parse_number_factory(ns.PATH | ns.UNGROUPLETTERS | ns.LOCALE, '', 'xx')(x) == ((('xx',), ('', x)),)



# -*- coding: utf-8 -*-
"""These test the utils.py functions."""
from __future__ import unicode_literals

import pytest
from natsort.compat.py23 import PY_VERSION
from natsort.ns_enum import ns
from natsort.utils import (
    _natsort_key,
    _regex_chooser,
    _parse_string_factory,
    _parse_path_factory,
    _parse_number_factory,
    _parse_bytes_factory,
    _input_string_transform_factory,
    _string_component_transform_factory,
    _final_data_transform_factory,
)
from hypothesis import (
    given,
)
from hypothesis.strategies import (
    lists,
    text,
    floats,
    integers,
    binary,
)

if PY_VERSION >= 3:
    long = int


regex = _regex_chooser[ns.INT]
pre = _input_string_transform_factory(ns.INT)
post = _string_component_transform_factory(ns.INT)
after = _final_data_transform_factory(ns.INT, '', '')
string_func = _parse_string_factory(ns.INT, '', regex.split, pre, post, after)
bytes_func = _parse_bytes_factory(ns.INT)
num_func = _parse_number_factory(ns.INT, '', '')


def test__natsort_key_with_numeric_input_and_PATH_returns_number_in_nested_tuple():
    # It gracefully handles as_path for numeric input by putting an extra tuple around it
    # so it will sort against the other as_path results.
    sfunc = _parse_path_factory(string_func)
    bytes_func = _parse_bytes_factory(ns.PATH)
    num_func = _parse_number_factory(ns.PATH, '', '')
    assert _natsort_key(10, None, sfunc, bytes_func, num_func) == (('', 10),)


@pytest.mark.skipif(PY_VERSION < 3, reason='only valid on python3')
def test__natsort_key_with_bytes_input_and_PATH_returns_number_in_nested_tuple():
    # It gracefully handles as_path for numeric input by putting an extra tuple around it
    # so it will sort against the other as_path results.
    sfunc = _parse_path_factory(string_func)
    bytes_func = _parse_bytes_factory(ns.PATH)
    num_func = _parse_number_factory(ns.PATH, '', '')
    assert _natsort_key(b'/hello/world', None, sfunc, bytes_func, num_func) == ((b'/hello/world',),)


def test__natsort_key_with_tuple_of_paths_and_PATH_returns_triply_nested_tuple():
    # PATH also handles recursion well.
    sfunc = _parse_path_factory(string_func)
    bytes_func = _parse_bytes_factory(ns.PATH)
    num_func = _parse_number_factory(ns.PATH, '', '')
    assert _natsort_key(('/Folder', '/Folder (1)'), None, sfunc, bytes_func, num_func) == ((('/',), ('Folder',)), (('/',), ('Folder (', 1, ')')))


# The remaining tests provide no examples, just hypothesis tests.
# They only confirm that _natsort_key uses the above building blocks.


@given(floats(allow_nan=False) | integers())
def test__natsort_key_with_numeric_input_takes_number_path(x):
    assert _natsort_key(x, None, string_func, bytes_func, num_func) == num_func(x)


@pytest.mark.skipif(PY_VERSION < 3, reason='only valid on python3')
@given(binary().filter(bool))
def test__natsort_key_with_bytes_input_takes_bytes_path(x):
    assert _natsort_key(x, None, string_func, bytes_func, num_func) == bytes_func(x)


@given(lists(elements=floats(allow_nan=False) | text() | integers(), min_size=1, max_size=10))
def test__natsort_key_with_text_input_takes_string_path(x):
    s = ''.join(repr(y) if type(y) in (float, long, int) else y for y in x)
    assert _natsort_key(s, None, string_func, bytes_func, num_func) == string_func(s)


@given(lists(elements=text(), min_size=1, max_size=10))
def test__natsort_key_with_nested_input_takes_nested_path(x):
    assert _natsort_key(x, None, string_func, bytes_func, num_func) == tuple(string_func(s) for s in x)


@given(text())
def test__natsort_key_with_key_argument_applies_key_before_processing(x):
    assert _natsort_key(x, len, string_func, bytes_func, num_func) == num_func(len(x))



# -*- coding: utf-8 -*-
"""These test the utils.py functions."""
from __future__ import unicode_literals

from natsort.ns_enum import ns
from natsort.utils import _parse_bytes_factory
from hypothesis import given
from hypothesis.strategies import binary


# Each test has an "example" version for demonstrative purposes,
# and a test that uses the hypothesis module.


def test_parse_bytes_factory_makes_function_that_returns_tuple_example():
    assert _parse_bytes_factory(0)(b'hello') == (b'hello',)


@given(binary())
def test_parse_bytes_factory_makes_function_that_returns_tuple(x):
    assert _parse_bytes_factory(0)(x) == (x,)


def test_parse_bytes_factory_with_IGNORECASE_makes_function_that_returns_tuple_with_lowercase_example():
    assert _parse_bytes_factory(ns.IGNORECASE)(b'HelLo') == (b'hello',)


@given(binary())
def test_parse_bytes_factory_with_IGNORECASE_makes_function_that_returns_tuple_with_lowercase(x):
    assert _parse_bytes_factory(ns.IGNORECASE)(x) == (x.lower(),)


def test_parse_bytes_factory_with_PATH_makes_function_that_returns_nested_tuple_example():
    assert _parse_bytes_factory(ns.PATH)(b'hello') == ((b'hello',),)


@given(binary())
def test_parse_bytes_factory_with_PATH_makes_function_that_returns_nested_tuple(x):
    assert _parse_bytes_factory(ns.PATH)(x) == ((x,),)


def test_parse_bytes_factory_with_PATH_and_IGNORECASE_makes_function_that_returns_nested_tuple_with_lowercase_example():
    assert _parse_bytes_factory(ns.PATH | ns.IGNORECASE)(b'HelLo') == ((b'hello',),)


@given(binary())
def test_parse_bytes_factory_with_PATH_and_IGNORECASE_makes_function_that_returns_nested_tuple_with_lowercase(x):
    assert _parse_bytes_factory(ns.PATH | ns.IGNORECASE)(x) == ((x.lower(),),)



# -*- coding: utf-8 -*-
"""These test the utils.py functions."""
from __future__ import unicode_literals

from natsort.ns_enum import ns
from natsort.utils import (
    _string_component_transform_factory,
    _groupletters,
)
from natsort.compat.py23 import py23_str
from natsort.compat.locale import get_strxfrm
from natsort.compat.fastnumbers import (
    fast_float,
    fast_int,
)
from hypothesis import (
    given,
)
from hypothesis.strategies import (
    text,
    floats,
    integers,
)
from compat.locale import bad_uni_chars


def no_null(x):
    return '\0' not in x


# Each test has an "example" version for demonstrative purposes,
# and a test that uses the hypothesis module.


def test_string_component_transform_factory_returns_fast_int_example():
    x = 'hello'
    assert _string_component_transform_factory(0)(x) is fast_int(x)
    assert _string_component_transform_factory(0)('5007') == fast_int('5007')


@given(text().filter(bool) | floats() | integers())
def test_string_component_transform_factory_returns_fast_int(x):
    assert _string_component_transform_factory(0)(py23_str(x)) == fast_int(py23_str(x))


def test_string_component_transform_factory_with_FLOAT_returns_fast_float_example():
    x = 'hello'
    assert _string_component_transform_factory(ns.FLOAT)(x) is fast_float(x)
    assert _string_component_transform_factory(ns.FLOAT)('5007') == fast_float('5007')


@given(text().filter(bool) | floats() | integers())
def test_string_component_transform_factory_with_FLOAT_returns_fast_float(x):
    assert _string_component_transform_factory(ns.FLOAT)(py23_str(x)) == fast_float(py23_str(x), nan=float('-inf'))


def test_string_component_transform_factory_with_FLOAT_returns_fast_float_with_neg_inf_replacing_nan():
    assert _string_component_transform_factory(ns.FLOAT)('nan') == fast_float('nan', nan=float('-inf'))


def test_string_component_transform_factory_with_FLOAT_and_NANLAST_returns_fast_float_with_pos_inf_replacing_nan():
    assert _string_component_transform_factory(ns.FLOAT | ns.NANLAST)('nan') == fast_float('nan', nan=float('+inf'))


def test_string_component_transform_factory_with_GROUPLETTERS_returns_fast_int_and_groupletters_example():
    x = 'hello'
    assert _string_component_transform_factory(ns.GROUPLETTERS)(x) == fast_int(x, key=_groupletters)


@given(text().filter(bool))
def test_string_component_transform_factory_with_GROUPLETTERS_returns_fast_int_and_groupletters(x):
    assert _string_component_transform_factory(ns.GROUPLETTERS)(x) == fast_int(x, key=_groupletters)


def test_string_component_transform_factory_with_LOCALE_returns_fast_int_and_groupletters_example():
    x = 'hello'
    assert _string_component_transform_factory(ns.LOCALE)(x) == fast_int(x, key=get_strxfrm())


@given(text().filter(bool).filter(lambda x: not any(y in bad_uni_chars for y in x)).filter(no_null))
def test_string_component_transform_factory_with_LOCALE_returns_fast_int_and_groupletters(x):
    assert _string_component_transform_factory(ns.LOCALE)(x) == fast_int(x, key=get_strxfrm())


def test_string_component_transform_factory_with_LOCALE_and_GROUPLETTERS_returns_fast_int_and_groupletters_and_locale_convert_example():
    x = 'hello'
    assert _string_component_transform_factory(ns.GROUPLETTERS | ns.LOCALE)(x) == fast_int(x, key=lambda x: get_strxfrm()(_groupletters(x)))


@given(text().filter(bool).filter(no_null))
def test_string_component_transform_factory_with_LOCALE_and_GROUPLETTERS_returns_fast_int_and_groupletters_and_locale_convert(x):
    try:
        assert _string_component_transform_factory(ns.GROUPLETTERS | ns.LOCALE)(x) == fast_int(x, key=lambda x: get_strxfrm()(_groupletters(x)))
    except ValueError as e:  # handle broken locale lib on BSD.
        if 'is not in range' not in str(e):
            raise


def test_string_component_transform_factory_with_LOCALE_and_DUMB_returns_fast_int_and_groupletters_and_locale_convert_example():
    x = 'hello'
    assert _string_component_transform_factory(ns._DUMB | ns.LOCALE)(x) == fast_int(x, key=lambda x: get_strxfrm()(_groupletters(x)))


@given(text().filter(bool).filter(no_null))
def test_string_component_transform_factory_with_LOCALE_and_DUMB_returns_fast_int_and_groupletters_and_locale_convert(x):
    try:
        assert _string_component_transform_factory(ns._DUMB | ns.LOCALE)(x) == fast_int(x, key=lambda x: get_strxfrm()(_groupletters(x)))
    except ValueError as e:  # handle broken locale lib on BSD.
        if 'is not in range' not in str(e):
            raise



# -*- coding: utf-8 -*-
"""These test the utils.py functions."""
from __future__ import unicode_literals

from natsort.ns_enum import ns
from natsort.utils import _final_data_transform_factory
from natsort.compat.py23 import py23_str
from hypothesis import (
    given,
)
from hypothesis.strategies import (
    text,
    floats,
    integers,
)


# Each test has an "example" version for demonstrative purposes,
# and a test that uses the hypothesis module.


def test_final_data_transform_factory_with_iterable_returns_tuple_with_no_options_example():
    assert _final_data_transform_factory(0, '', '')(iter([7]), '') == (7,)


@given(text())
def test_final_data_transform_factory_with_iterable_returns_tuple_with_no_options(x):
    assert _final_data_transform_factory(0, '', '')(iter([x]), '') == (x,)
    # UNGROUPLETTERS without LOCALE does nothing, as does LOCALE without UNGROUPLETTERS
    assert _final_data_transform_factory(ns.UNGROUPLETTERS, '', '')(iter([x]), '') == _final_data_transform_factory(0, '', '')(iter([x]), '')
    assert _final_data_transform_factory(ns.LOCALE, '', '')(iter([x]), '') == _final_data_transform_factory(0, '', '')(iter([x]), '')


def test_final_data_transform_factory_with_empty_tuple_returns_double_empty_tuple():
    assert _final_data_transform_factory(ns.LOCALE | ns.UNGROUPLETTERS, '', '')((), '') == ((), ())


def test_final_data_transform_factory_with_null_string_first_element_adds_empty_string_on_first_tuple_element():
    assert _final_data_transform_factory(ns.LOCALE | ns.UNGROUPLETTERS, '', 'xx')(('', 60), '') == (('xx',), ('', 60))


def test_final_data_transform_factory_returns_first_element_in_first_tuple_element_example():
    assert _final_data_transform_factory(ns.LOCALE | ns.UNGROUPLETTERS, '', '')(('this', 60), 'this60') == (('t',), ('this', 60))


@given(x=text().filter(bool), y=floats(allow_nan=False, allow_infinity=False) | integers())
def test_final_data_transform_factory_returns_first_element_in_first_tuple_element(x, y):
    assert _final_data_transform_factory(ns.LOCALE | ns.UNGROUPLETTERS, '', '')((x, y), ''.join(map(py23_str, [x, y]))) == ((x[0],), (x, y))


def test_final_data_transform_factory_returns_first_element_in_first_tuple_element_caseswapped_with_DUMB_and_LOWERCASEFIRST_example():
    assert _final_data_transform_factory(ns.LOCALE | ns.UNGROUPLETTERS | ns._DUMB | ns.LOWERCASEFIRST, '', '')(('this', 60), 'this60') == (('T',), ('this', 60))


@given(x=text().filter(bool), y=floats(allow_nan=False, allow_infinity=False) | integers())
def test_final_data_transform_factory_returns_first_element_in_first_tuple_element_caseswapped_with_DUMB_and_LOWERCASEFIRST(x, y):
    assert _final_data_transform_factory(ns.LOCALE | ns.UNGROUPLETTERS | ns._DUMB | ns.LOWERCASEFIRST, '', '')((x, y), ''.join(map(py23_str, [x, y]))) == ((x[0].swapcase(),), (x, y))



# -*- coding: utf-8 -*-
"""\
Here are a collection of examples of how this module can be used.
See the README or the natsort homepage for more details.
"""
from __future__ import unicode_literals, print_function
import pytest
import locale
from natsort.compat.py23 import PY_VERSION
from operator import itemgetter
from pytest import raises
from natsort import (
    natsorted,
    ns,
)
from compat.locale import (
    load_locale,
    has_locale_de_DE,
)


def test_natsorted_returns_strings_with_numbers_in_ascending_order():
    a = ['a2', 'a5', 'a9', 'a1', 'a4', 'a10', 'a6']
    assert natsorted(a) == ['a1', 'a2', 'a4', 'a5', 'a6', 'a9', 'a10']


def test_natsorted_returns_list_of_numbers_sorted_as_signed_floats_with_exponents():
    a = ['a50', 'a51.', 'a50.31', 'a-50', 'a50.4', 'a5.034e1', 'a50.300']
    assert natsorted(a, alg=ns.REAL) == ['a-50', 'a50', 'a50.300', 'a50.31', 'a5.034e1', 'a50.4', 'a51.']


def test_natsorted_returns_list_of_numbers_sorted_as_unsigned_floats_without_exponents_with_NOEXP_option():
    a = ['a50', 'a51.', 'a50.31', 'a-50', 'a50.4', 'a5.034e1', 'a50.300']
    assert natsorted(a, alg=ns.N | ns.F | ns.U) == ['a5.034e1', 'a50', 'a50.300', 'a50.31', 'a50.4', 'a51.', 'a-50']
    # UNSIGNED is default
    assert natsorted(a, alg=ns.NOEXP | ns.FLOAT) == ['a5.034e1', 'a50', 'a50.300', 'a50.31', 'a50.4', 'a51.', 'a-50']


def test_natsorted_returns_list_of_numbers_sorted_as_unsigned_ints_with_INT_option():
    a = ['a50', 'a51.', 'a50.31', 'a-50', 'a50.4', 'a5.034e1', 'a50.300']
    assert natsorted(a, alg=ns.INT) == ['a5.034e1', 'a50', 'a50.4', 'a50.31', 'a50.300', 'a51.', 'a-50']
    # INT is default
    assert natsorted(a) == ['a5.034e1', 'a50', 'a50.4', 'a50.31', 'a50.300', 'a51.', 'a-50']


def test_natsorted_returns_list_of_numbers_sorted_as_unsigned_ints_with_DIGIT_and_VERSION_option():
    a = ['a50', 'a51.', 'a50.31', 'a-50', 'a50.4', 'a5.034e1', 'a50.300']
    assert natsorted(a, alg=ns.DIGIT) == ['a5.034e1', 'a50', 'a50.4', 'a50.31', 'a50.300', 'a51.', 'a-50']
    assert natsorted(a, alg=ns.VERSION) == ['a5.034e1', 'a50', 'a50.4', 'a50.31', 'a50.300', 'a51.', 'a-50']


def test_natsorted_returns_list_of_numbers_sorted_as_signed_ints_with_SIGNED_option():
    a = ['a50', 'a51.', 'a50.31', 'a-50', 'a50.4', 'a5.034e1', 'a50.300']
    assert natsorted(a, alg=ns.SIGNED) == ['a-50', 'a5.034e1', 'a50', 'a50.4', 'a50.31', 'a50.300', 'a51.']


def test_natsorted_returns_list_of_numbers_sorted_accounting_for_sign_with_SIGNED_option():
    a = ['a-5', 'a7', 'a+2']
    assert natsorted(a, alg=ns.SIGNED) == ['a-5', 'a+2', 'a7']


def test_natsorted_returns_list_of_numbers_sorted_not_accounting_for_sign_without_SIGNED_option():
    a = ['a-5', 'a7', 'a+2']
    assert natsorted(a) == ['a7', 'a+2', 'a-5']


def test_natsorted_returns_sorted_list_of_version_numbers_by_default_or_with_VERSION_option():
    a = ['1.9.9a', '1.11', '1.9.9b', '1.11.4', '1.10.1']
    assert natsorted(a) == ['1.9.9a', '1.9.9b', '1.10.1', '1.11', '1.11.4']
    assert natsorted(a, alg=ns.VERSION) == ['1.9.9a', '1.9.9b', '1.10.1', '1.11', '1.11.4']


def test_natsorted_returns_sorted_list_with_mixed_type_input_and_does_not_raise_TypeError_on_Python3():
    # You can mix types with natsorted.  This can get around the new
    # 'unorderable types' issue with Python 3.
    a = [6, 4.5, '7', '2.5', 'a']
    assert natsorted(a) == ['2.5', 4.5, 6, '7', 'a']
    a = [46, '5a5b2', 'af5', '5a5-4']
    assert natsorted(a) == ['5a5-4', '5a5b2', 46, 'af5']


def test_natsorted_with_mixed_input_returns_sorted_results_without_error():
    a = ['0', 'Á', '2', 'Z']
    assert natsorted(a) == ['0', '2', 'Á', 'Z']
    assert natsorted(a, alg=ns.NUMAFTER) == ['Á', 'Z', '0', '2']
    a = ['2', 'ä', 'b', 1.5, 3]
    assert natsorted(a) == [1.5, '2', 3, 'ä', 'b']
    assert natsorted(a, alg=ns.NUMAFTER) == ['ä', 'b', 1.5, '2', 3]


def test_natsorted_with_nan_input_returns_sorted_results_with_nan_last_with_NANLAST():
    a = ['25', 5, float('nan'), 1E40]
    # The slice is because NaN != NaN
    assert natsorted(a, alg=ns.NANLAST)[:3] == [5, '25', 1E40, float('nan')][:3]


def test_natsorted_with_nan_input_returns_sorted_results_with_nan_first_without_NANLAST():
    a = ['25', 5, float('nan'), 1E40]
    # The slice is because NaN != NaN
    assert natsorted(a)[1:] == [float('nan'), 5, '25', 1E40][1:]


def test_natsorted_with_mixed_input_raises_TypeError_if_bytes_type_is_involved_on_Python3():
    if PY_VERSION >= 3:
        with raises(TypeError) as e:
            assert natsorted(['ä', b'b'])
        assert 'bytes' in str(e.value)
    else:
        assert True


def test_natsorted_raises_ValueError_for_non_iterable_input():
    with raises(TypeError) as err:
        natsorted(100)
    assert str(err.value) == "'int' object is not iterable"


def test_natsorted_recursivley_applies_key_to_nested_lists_to_return_sorted_nested_list():
    data = [['a1', 'a5'], ['a1', 'a40'], ['a10', 'a1'], ['a2', 'a5']]
    assert natsorted(data) == [['a1', 'a5'], ['a1', 'a40'], ['a2', 'a5'], ['a10', 'a1']]


def test_natsorted_applies_key_to_each_list_element_before_sorting_list():
    b = [('a', 'num3'), ('b', 'num5'), ('c', 'num2')]
    assert natsorted(b, key=itemgetter(1)) == [('c', 'num2'), ('a', 'num3'), ('b', 'num5')]


def test_natsorted_returns_list_in_reversed_order_with_reverse_option():
    a = ['a50', 'a51.', 'a50.31', 'a50.4', 'a5.034e1', 'a50.300']
    assert natsorted(a, reverse=True) == natsorted(a)[::-1]


def test_natsorted_sorts_OS_generated_paths_incorrectly_without_PATH_option():
    a = ['/p/Folder (10)/file.tar.gz',
         '/p/Folder/file.tar.gz',
         '/p/Folder (1)/file (1).tar.gz',
         '/p/Folder (1)/file.tar.gz']
    assert natsorted(a) == ['/p/Folder (1)/file (1).tar.gz',
                            '/p/Folder (1)/file.tar.gz',
                            '/p/Folder (10)/file.tar.gz',
                            '/p/Folder/file.tar.gz']


def test_natsorted_sorts_OS_generated_paths_correctly_with_PATH_option():
    a = ['/p/Folder (10)/file.tar.gz',
         '/p/Folder/file.tar.gz',
         '/p/Folder (1)/file (1).tar.gz',
         '/p/Folder (1)/file.tar.gz']
    assert natsorted(a, alg=ns.PATH) == ['/p/Folder/file.tar.gz',
                                         '/p/Folder (1)/file.tar.gz',
                                         '/p/Folder (1)/file (1).tar.gz',
                                         '/p/Folder (10)/file.tar.gz']


def test_natsorted_can_handle_sorting_paths_and_numbers_with_PATH():
    # You can sort paths and numbers, not that you'd want to
    a = ['/Folder (9)/file.exe', 43]
    assert natsorted(a, alg=ns.PATH) == [43, '/Folder (9)/file.exe']


def test_natsorted_returns_results_in_ASCII_order_with_no_case_options():
    a = ['Apple', 'corn', 'Corn', 'Banana', 'apple', 'banana']
    assert natsorted(a) == ['Apple', 'Banana', 'Corn', 'apple', 'banana', 'corn']


def test_natsorted_returns_results_sorted_by_lowercase_ASCII_order_with_IGNORECASE():
    a = ['Apple', 'corn', 'Corn', 'Banana', 'apple', 'banana']
    assert natsorted(a, alg=ns.IGNORECASE) == ['Apple', 'apple', 'Banana', 'banana', 'corn', 'Corn']


def test_natsorted_returns_results_in_ASCII_order_but_with_lowercase_letters_first_with_LOWERCASEFIRST():
    a = ['Apple', 'corn', 'Corn', 'Banana', 'apple', 'banana']
    assert natsorted(a, alg=ns.LOWERCASEFIRST) == ['apple', 'banana', 'corn', 'Apple', 'Banana', 'Corn']


def test_natsorted_returns_results_with_uppercase_and_lowercase_letters_grouped_together_with_GROUPLETTERS():
    a = ['Apple', 'corn', 'Corn', 'Banana', 'apple', 'banana']
    assert natsorted(a, alg=ns.GROUPLETTERS) == ['Apple', 'apple', 'Banana', 'banana', 'Corn', 'corn']


def test_natsorted_returns_results_in_natural_order_with_GROUPLETTERS_and_LOWERCASEFIRST():
    a = ['Apple', 'corn', 'Corn', 'Banana', 'apple', 'banana']
    assert natsorted(a, alg=ns.G | ns.LF) == ['apple', 'Apple', 'banana', 'Banana', 'corn', 'Corn']


def test_natsorted_places_uppercase_letters_before_lowercase_letters_for_nested_input():
    b = [('A5', 'a6'), ('a3', 'a1')]
    assert natsorted(b) == [('A5', 'a6'), ('a3', 'a1')]


def test_natsorted_with_LOWERCASEFIRST_places_lowercase_letters_before_uppercase_letters_for_nested_input():
    b = [('A5', 'a6'), ('a3', 'a1')]
    assert natsorted(b, alg=ns.LOWERCASEFIRST) == [('a3', 'a1'), ('A5', 'a6')]


def test_natsorted_with_IGNORECASE_sorts_without_regard_to_case_for_nested_input():
    b = [('A5', 'a6'), ('a3', 'a1')]
    assert natsorted(b, alg=ns.IGNORECASE) == [('a3', 'a1'), ('A5', 'a6')]


def test_natsorted_with_LOCALE_returns_results_sorted_by_lowercase_first_and_grouped_letters():
    a = ['Apple', 'corn', 'Corn', 'Banana', 'apple', 'banana']
    load_locale('en_US')
    assert natsorted(a, alg=ns.LOCALE) == ['apple', 'Apple', 'banana', 'Banana', 'corn', 'Corn']
    locale.setlocale(locale.LC_ALL, str(''))


def test_natsorted_with_LOCALE_and_CAPITALFIRST_returns_results_sorted_by_capital_first_and_ungrouped():
    a = ['Apple', 'corn', 'Corn', 'Banana', 'apple', 'banana']
    load_locale('en_US')
    assert natsorted(a, alg=ns.LOCALE | ns.CAPITALFIRST) == ['Apple', 'Banana', 'Corn', 'apple', 'banana', 'corn']
    locale.setlocale(locale.LC_ALL, str(''))


def test_natsorted_with_LOCALE_and_LOWERCASEFIRST_returns_results_sorted_by_uppercase_first_and_grouped_letters():
    a = ['Apple', 'corn', 'Corn', 'Banana', 'apple', 'banana']
    load_locale('en_US')
    assert natsorted(a, alg=ns.LOCALE | ns.LOWERCASEFIRST) == ['Apple', 'apple', 'Banana', 'banana', 'Corn', 'corn']
    locale.setlocale(locale.LC_ALL, str(''))


def test_natsorted_with_LOCALE_and_CAPITALFIRST_and_LOWERCASE_returns_results_sorted_by_capital_last_and_ungrouped():
    a = ['Apple', 'corn', 'Corn', 'Banana', 'apple', 'banana']
    load_locale('en_US')
    assert natsorted(a, alg=ns.LOCALE | ns.CAPITALFIRST | ns.LOWERCASEFIRST) == ['apple', 'banana', 'corn', 'Apple', 'Banana', 'Corn']
    locale.setlocale(locale.LC_ALL, str(''))


def test_natsorted_with_LOCALE_and_en_setting_returns_results_sorted_by_en_language():
    load_locale('en_US')
    a = ['c', 'a5,467.86', 'ä', 'b', 'a5367.86', 'a5,6', 'a5,50']
    assert natsorted(a, alg=ns.LOCALE | ns.F) == ['a5,6', 'a5,50', 'a5367.86', 'a5,467.86', 'ä', 'b', 'c']
    locale.setlocale(locale.LC_ALL, str(''))


@pytest.mark.skipif(not has_locale_de_DE, reason='requires de_DE locale and working locale')
def test_natsorted_with_LOCALE_and_de_setting_returns_results_sorted_by_de_language():
    load_locale('de_DE')
    a = ['c', 'a5.467,86', 'ä', 'b', 'a5367.86', 'a5,6', 'a5,50']
    assert natsorted(a, alg=ns.LOCALE | ns.F) == ['a5,50', 'a5,6', 'a5367.86', 'a5.467,86', 'ä', 'b', 'c']
    locale.setlocale(locale.LC_ALL, str(''))


def test_natsorted_with_LOCALE_and_mixed_input_returns_sorted_results_without_error():
    load_locale('en_US')
    a = ['0', 'Á', '2', 'Z']
    assert natsorted(a, alg=ns.LOCALE) == ['0', '2', 'Á', 'Z']
    assert natsorted(a, alg=ns.LOCALE | ns.NUMAFTER) == ['Á', 'Z', '0', '2']
    a = ['2', 'ä', 'b', 1.5, 3]
    assert natsorted(a, alg=ns.LOCALE) == [1.5, '2', 3, 'ä', 'b']
    assert natsorted(a, alg=ns.LOCALE | ns.NUMAFTER) == ['ä', 'b', 1.5, '2', 3]
    locale.setlocale(locale.LC_ALL, str(''))


def test_natsorted_with_LOCALE_and_UNGROUPLETTERS_and_mixed_input_returns_sorted_results_without_error():
    load_locale('en_US')
    a = ['0', 'Á', '2', 'Z']
    assert natsorted(a, alg=ns.LOCALE | ns.UNGROUPLETTERS) == ['0', '2', 'Á', 'Z']
    assert natsorted(a, alg=ns.LOCALE | ns.UNGROUPLETTERS | ns.NUMAFTER) == ['Á', 'Z', '0', '2']
    a = ['2', 'ä', 'b', 1.5, 3]
    assert natsorted(a, alg=ns.LOCALE | ns.UNGROUPLETTERS) == [1.5, '2', 3, 'ä', 'b']
    assert natsorted(a, alg=ns.LOCALE | ns.UNGROUPLETTERS | ns.NUMAFTER) == ['ä', 'b', 1.5, '2', 3]
    locale.setlocale(locale.LC_ALL, str(''))


def test_natsorted_with_PATH_and_LOCALE_and_UNGROUPLETTERS_and_mixed_input_returns_sorted_results_without_error():
    load_locale('en_US')
    a = ['0', 'Á', '2', 'Z']
    assert natsorted(a, alg=ns.PATH | ns.LOCALE | ns.UNGROUPLETTERS) == ['0', '2', 'Á', 'Z']
    assert natsorted(a, alg=ns.PATH | ns.LOCALE | ns.UNGROUPLETTERS | ns.NUMAFTER) == ['Á', 'Z', '0', '2']
    a = ['2', 'ä', 'b', 1.5, 3]
    assert natsorted(a, alg=ns.PATH | ns.LOCALE | ns.UNGROUPLETTERS) == [1.5, '2', 3, 'ä', 'b']
    assert natsorted(a, alg=ns.PATH | ns.LOCALE | ns.UNGROUPLETTERS | ns.NUMAFTER) == ['ä', 'b', 1.5, '2', 3]
    locale.setlocale(locale.LC_ALL, str(''))


def test_natsorted_sorts_an_odd_collection_of_string():
    a = ['Corn', 'apple', 'Banana', '73', 'Apple', '5039', 'corn', '~~~~~~', 'banana']
    assert natsorted(a) == ['73', '5039', 'Apple', 'Banana', 'Corn',
                            'apple', 'banana', 'corn', '~~~~~~']
    assert natsorted(a, alg=ns.NUMAFTER) == ['Apple', 'Banana', 'Corn',
                                             'apple', 'banana', 'corn', '~~~~~~', '73', '5039']


def test_natsorted_sorts_mixed_ascii_and_non_ascii_numbers():
    a = ['1st street', '10th street', '2nd street', '2 street', '1 street', '1street',
         '11 street', 'street 2', 'street 1', 'Street 11', '۲ street', '۱ street', '۱street',
         '۱۲street', '۱۱ street', 'street ۲', 'street ۱', 'street ۱', 'street ۱۲', 'street ۱۱']
    expected = ['1 street', '۱ street', '1st street', '1street', '۱street', '2 street', '۲ street',
                '2nd street', '10th street', '11 street', '۱۱ street', '۱۲street', 'street 1',
                'street ۱', 'street ۱', 'street 2', 'street ۲', 'Street 11', 'street ۱۱', 'street ۱۲']
    assert natsorted(a, alg=ns.IGNORECASE) == expected



# -*- coding: utf-8 -*-
"""\
Test the Unicode numbers module.
"""
from __future__ import unicode_literals
import unicodedata
from natsort.compat.py23 import py23_range, py23_unichr
from natsort.unicode_numbers import (
    numeric_hex,
    numeric_chars,
    numeric,
    digit_chars,
    digits,
    decimal_chars,
    decimals,
    digits_no_decimals,
    numeric_no_decimals,
)


def test_numeric_chars_contains_only_valid_unicode_numeric_characters():
    for a in numeric_chars:
        assert unicodedata.numeric(a, None) is not None


def test_digit_chars_contains_only_valid_unicode_digit_characters():
    for a in digit_chars:
        assert unicodedata.digit(a, None) is not None


def test_decimal_chars_contains_only_valid_unicode_decimal_characters():
    for a in decimal_chars:
        assert unicodedata.decimal(a, None) is not None


def test_numeric_chars_contains_all_valid_unicode_numeric_and_digit_characters():
    set_numeric_hex = set(numeric_hex)
    set_numeric_chars = set(numeric_chars)
    set_digit_chars = set(digit_chars)
    set_decimal_chars = set(decimal_chars)
    for i in py23_range(0X110000):
        try:
            a = py23_unichr(i)
        except ValueError:
            break
        if a in set('0123456789'):
            continue
        if unicodedata.numeric(a, None) is not None:
            assert i in set_numeric_hex
            assert a in set_numeric_chars
        if unicodedata.digit(a, None) is not None:
            assert i in set_numeric_hex
            assert a in set_digit_chars
        if unicodedata.decimal(a, None) is not None:
            assert i in set_numeric_hex
            assert a in set_decimal_chars

    assert set_decimal_chars.isdisjoint(digits_no_decimals)
    assert set_digit_chars.issuperset(digits_no_decimals)

    assert set_decimal_chars.isdisjoint(numeric_no_decimals)
    assert set_numeric_chars.issuperset(numeric_no_decimals)


def test_combined_string_contains_all_characters_in_list():
    assert numeric == ''.join(numeric_chars)
    assert digits == ''.join(digit_chars)
    assert decimals == ''.join(decimal_chars)



# -*- coding: utf-8 -*-
"""\
Here are a collection of examples of how this module can be used.
See the README or the natsort homepage for more details.
"""
from __future__ import unicode_literals, print_function

import warnings
import locale
from pytest import raises
from natsort import (
    natsorted,
    natsort_key,
    natsort_keygen,
    ns,
)
from natsort.compat.py23 import PY_VERSION
from natsort.compat.locale import (
    null_string_locale,
    get_strxfrm,
)
from compat.mock import patch
from compat.locale import load_locale

INPUT = ['6A-5.034e+1', '/Folder (1)/Foo', 56.7]


def test_natsort_key_public_raises_DeprecationWarning_when_called():
    # But it raises a deprecation warning
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")
        assert natsort_key('a-5.034e2') == ('a-', 5, '.', 34, 'e', 2)
        assert len(w) == 1
        assert "natsort_key is deprecated as of 3.4.0, please use natsort_keygen" in str(w[-1].message)
    # It is called for each element in a list when sorting
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")
        a = ['a2', 'a5', 'a9', 'a1', 'a4', 'a10', 'a6']
        a.sort(key=natsort_key)
        assert len(w) == 7


def test_natsort_keygen_with_invalid_alg_input_raises_ValueError():
    # Invalid arguments give the correct response
    with raises(ValueError) as err:
        natsort_keygen(None, '1')
    assert str(err.value) == "natsort_keygen: 'alg' argument must be from the enum 'ns', got 1"


def test_natsort_keygen_returns_natsort_key_that_parses_input():
    a = 'a-5.034e1'
    assert natsort_keygen()(a) == ('a-', 5, '.', 34, 'e', 1)
    assert natsort_keygen(alg=ns.F | ns.S)(a) == ('a', -50.34)


def test_natsort_keygen_returns_key_that_can_be_used_to_sort_list_in_place_with_same_result_as_natsorted():
    a = ['a50', 'a51.', 'a50.31', 'a50.4', 'a5.034e1', 'a50.300']
    b = a[:]
    a.sort(key=natsort_keygen(alg=ns.F))
    assert a == natsorted(b, alg=ns.F)


def test_natsort_keygen_splits_input_with_defaults():
    assert natsort_keygen()(INPUT) == (('', 6, 'A-', 5, '.', 34, 'e+', 1), ('/Folder (', 1, ')/Foo'), ('', 56.7))
    if PY_VERSION >= 3: assert natsort_keygen()(b'6A-5.034e+1') == (b'6A-5.034e+1',)


def test_natsort_keygen_splits_input_with_real():
    assert natsort_keygen(alg=ns.R)(INPUT) == (('', 6.0, 'A', -50.34), ('/Folder (', 1.0, ')/Foo'), ('', 56.7))
    if PY_VERSION >= 3: assert natsort_keygen(alg=ns.R)(b'6A-5.034e+1') == (b'6A-5.034e+1',)


def test_natsort_keygen_splits_input_with_lowercasefirst_noexp_float():
    assert natsort_keygen(alg=ns.LF | ns.F | ns.N)(INPUT) == (('', 6.0, 'a-', 5.034, 'E+', 1.0), ('/fOLDER (', 1.0, ')/fOO'), ('', 56.7))
    if PY_VERSION >= 3: assert natsort_keygen(alg=ns.LF | ns.F | ns.N)(b'6A-5.034e+1') == (b'6A-5.034e+1',)


def test_natsort_keygen_splits_input_with_locale():
    load_locale('en_US')
    strxfrm = get_strxfrm()
    with patch('natsort.compat.locale.dumb_sort', return_value=False):
        assert natsort_keygen(alg=ns.L)(INPUT) == ((null_string_locale, 6, strxfrm('A-'), 5, strxfrm('.'), 34, strxfrm('e+'), 1), (strxfrm('/Folder ('), 1, strxfrm(')/Foo')), (null_string_locale, 56.7))
    with patch('natsort.compat.locale.dumb_sort', return_value=True):
        assert natsort_keygen(alg=ns.L)(INPUT) == ((null_string_locale, 6, strxfrm('aa--'), 5, strxfrm('..'), 34, strxfrm('eE++'), 1), (strxfrm('//ffoOlLdDeErR  (('), 1, strxfrm('))//ffoOoO')), (null_string_locale, 56.7))
    if PY_VERSION >= 3: assert natsort_keygen(alg=ns.LA)(b'6A-5.034e+1') == (b'6A-5.034e+1',)
    locale.setlocale(locale.LC_ALL, str(''))


def test_natsort_keygen_splits_input_with_locale_and_capitalfirst():
    load_locale('en_US')
    strxfrm = get_strxfrm()
    with patch('natsort.compat.locale.dumb_sort', return_value=False):
        assert natsort_keygen(alg=ns.LA | ns.C)(INPUT) == ((('',), (null_string_locale, 6, strxfrm('A-'), 5, strxfrm('.'), 34, strxfrm('e+'), 1)), (('/',), (strxfrm('/Folder ('), 1, strxfrm(')/Foo'))), (('',), (null_string_locale, 56.7)))
    if PY_VERSION >= 3: assert natsort_keygen(alg=ns.LA | ns.C)(b'6A-5.034e+1') == (b'6A-5.034e+1',)
    locale.setlocale(locale.LC_ALL, str(''))


def test_natsort_keygen_splits_input_with_path():
    assert natsort_keygen(alg=ns.P | ns.G)(INPUT) == ((('', 6, 'aA--', 5, '..', 34, 'ee++', 1),), (('//',), ('fFoollddeerr  ((', 1, '))'), ('fFoooo',)), (('', 56.7),))
    if PY_VERSION >= 3: assert natsort_keygen(alg=ns.P | ns.G)(b'6A-5.034e+1') == ((b'6A-5.034e+1',),)


def test_natsort_keygen_splits_input_with_ignorecase():
    assert natsort_keygen(alg=ns.IC)(INPUT) == (('', 6, 'a-', 5, '.', 34, 'e+', 1), ('/folder (', 1, ')/foo'), ('', 56.7))
    if PY_VERSION >= 3: assert natsort_keygen(alg=ns.IC)(b'6A-5.034e+1') == (b'6a-5.034e+1',)



#! /usr/bin/env python

from setuptools import setup, find_packages
setup(
    name='natsort',
    version='5.3.3',
    packages=find_packages(),
    install_requires=["argparse; python_version < '2.7'"],
    entry_points={'console_scripts': ['natsort = natsort.__main__:main']},
    extras_require={
        'fast': ["fastnumbers >= 2.0.0; python_version > '2.6'"],
        'icu': ["PyICU >= 1.0.0"]
    }
)



# -*- coding: utf-8 -*-
# pylint: disable=unused-variable
"""These test the natcmp() function.

Note that these tests are only relevant for Python version < 3.
"""
import sys
from functools import partial
from compat.mock import patch

import pytest
from hypothesis import given
from hypothesis.strategies import floats, integers, lists

from natsort import ns

from natsort.compat.py23 import py23_cmp

PY_VERSION = float(sys.version[:3])

if PY_VERSION < 3:
    from natsort import natcmp


class Comparable(object):
    """Stub class for testing natcmp functionality."""
    def __init__(self, value):
        self.value = value

    def __cmp__(self, other):
        return natcmp(self.value, other.value)


@pytest.mark.skipif(PY_VERSION >= 3.0, reason='cmp() deprecated in Python 3')
def test__classes_can_be_compared():
    one = Comparable("1")
    two = Comparable("2")
    another_two = Comparable("2")
    ten = Comparable("10")

    assert ten > two == another_two > one


@pytest.mark.skipif(PY_VERSION >= 3.0, reason='cmp() deprecated in Python 3')
def test__keys_are_being_cached():
    natcmp.cached_keys = {}
    assert len(natcmp.cached_keys) == 0
    natcmp(0, 0)
    assert len(natcmp.cached_keys) == 1
    natcmp(0, 0)
    assert len(natcmp.cached_keys) == 1

    with patch('natsort.compat.locale.dumb_sort', return_value=False):
        natcmp(0, 0, alg=ns.L)
        assert len(natcmp.cached_keys) == 2
        natcmp(0, 0, alg=ns.L)
        assert len(natcmp.cached_keys) == 2

    with patch('natsort.compat.locale.dumb_sort', return_value=True):
        natcmp(0, 0, alg=ns.L)
        assert len(natcmp.cached_keys) == 3
        natcmp(0, 0, alg=ns.L)
        assert len(natcmp.cached_keys) == 3


@pytest.mark.skipif(PY_VERSION >= 3.0, reason='cmp() deprecated in Python 3')
def test__illegal_algorithm_raises_error():
    try:
        natcmp(0, 0, alg="Just random stuff")
        assert False

    except ValueError:
        assert True

    except Exception:
        assert False


@pytest.mark.skipif(PY_VERSION >= 3.0, reason='cmp() deprecated in Python 3')
def test__classes_can_utilize_max_or_min():
    comparables = [Comparable(i) for i in range(10)]

    assert max(comparables) == comparables[-1]
    assert min(comparables) == comparables[0]


@pytest.mark.skipif(PY_VERSION >= 3.0, reason='cmp() deprecated in Python 3')
@given(integers(), integers())
def test__natcmp_works_the_same_for_integers_as_cmp(x, y):
    assert py23_cmp(x, y) == natcmp(x, y)


@pytest.mark.skipif(PY_VERSION >= 3.0, reason='cmp() deprecated in Python 3')
@given(floats(allow_nan=False), floats(allow_nan=False))
def test__natcmp_works_the_same_for_floats_as_cmp(x, y):
    assert py23_cmp(x, y) == natcmp(x, y)


@pytest.mark.skipif(PY_VERSION >= 3.0, reason='cmp() deprecated in Python 3')
@given(lists(elements=integers()))
def test_sort_strings_with_numbers(a_list):
    strings = [str(var) for var in a_list]
    natcmp_sorted = sorted(strings, cmp=partial(natcmp, alg=ns.SIGNED))

    assert sorted(a_list) == [int(var) for var in natcmp_sorted]



# -*- coding: utf-8 -*-
from __future__ import (
    print_function,
    division,
    unicode_literals,
    absolute_import
)
# Load mock functions from the right place.
try:
    from unittest.mock import MagicMock, patch, call
except ImportError:
    from mock import MagicMock, patch, call



# -*- coding: utf-8 -*-
"""\
Test the natsort command-line tool functions.
"""
from __future__ import print_function, unicode_literals
import re
import sys
from pytest import raises
from compat.mock import patch, call
from hypothesis import (
    given,
)
from hypothesis.strategies import (
    integers,
    floats,
    lists,
    data,
)
from natsort.__main__ import (
    main,
    range_check,
    check_filter,
    keep_entry_range,
    exclude_entry,
    sort_and_print_entries,
)


def test_main_passes_default_arguments_with_no_command_line_options():
    with patch('natsort.__main__.sort_and_print_entries') as p:
        sys.argv[1:] = ['num-2', 'num-6', 'num-1']
        main()
        args = p.call_args[0][1]
        assert not args.paths
        assert args.filter is None
        assert args.reverse_filter is None
        assert args.exclude is None
        assert not args.reverse
        assert args.number_type == 'int'
        assert not args.signed
        assert args.exp
        assert not args.locale


def test_main_passes_arguments_with_all_command_line_options():
    with patch('natsort.__main__.sort_and_print_entries') as p:
        sys.argv[1:] = ['--paths', '--reverse', '--locale',
                        '--filter', '4', '10',
                        '--reverse-filter', '100', '110',
                        '--number-type', 'float', '--noexp', '--sign',
                        '--exclude', '34', '--exclude', '35',
                        'num-2', 'num-6', 'num-1']
        main()
        args = p.call_args[0][1]
        assert args.paths
        assert args.filter == [(4.0, 10.0)]
        assert args.reverse_filter == [(100.0, 110.0)]
        assert args.exclude == [34, 35]
        assert args.reverse
        assert args.number_type == 'float'
        assert args.signed
        assert not args.exp
        assert args.locale


class Args:
    """A dummy class to simulate the argparse Namespace object"""
    def __init__(self, filt, reverse_filter, exclude, as_path, reverse):
        self.filter = filt
        self.reverse_filter = reverse_filter
        self.exclude = exclude
        self.reverse = reverse
        self.number_type = 'float'
        self.signed = True
        self.exp = True
        self.paths = as_path
        self.locale = 0

entries = ['tmp/a57/path2',
           'tmp/a23/path1',
           'tmp/a1/path1',
           'tmp/a1 (1)/path1',
           'tmp/a130/path1',
           'tmp/a64/path1',
           'tmp/a64/path2']

mock_print = '__builtin__.print' if sys.version[0] == '2' else 'builtins.print'


def test_sort_and_print_entries_uses_default_algorithm_with_all_options_false():
    with patch(mock_print) as p:
        # tmp/a1 (1)/path1
        # tmp/a1/path1
        # tmp/a23/path1
        # tmp/a57/path2
        # tmp/a64/path1
        # tmp/a64/path2
        # tmp/a130/path1
        sort_and_print_entries(entries, Args(None, None, False, False, False))
        e = [call(entries[i]) for i in [3, 2, 1, 0, 5, 6, 4]]
        p.assert_has_calls(e)


def test_sort_and_print_entries_uses_PATH_algorithm_with_path_option_true_to_properly_sort_OS_generated_path_names():
    with patch(mock_print) as p:
        # tmp/a1/path1
        # tmp/a1 (1)/path1
        # tmp/a23/path1
        # tmp/a57/path2
        # tmp/a64/path1
        # tmp/a64/path2
        # tmp/a130/path1
        sort_and_print_entries(entries, Args(None, None, False, True, False))
        e = [call(entries[i]) for i in [2, 3, 1, 0, 5, 6, 4]]
        p.assert_has_calls(e)


def test_sort_and_print_entries_keeps_only_paths_between_of_20_to_100_with_filter_option():
    with patch(mock_print) as p:
        # tmp/a23/path1
        # tmp/a57/path2
        # tmp/a64/path1
        # tmp/a64/path2
        sort_and_print_entries(entries, Args([(20, 100)], None, False, False, False))
        e = [call(entries[i]) for i in [1, 0, 5, 6]]
        p.assert_has_calls(e)


def test_sort_and_print_entries_excludes_paths_between_of_20_to_100_with_reverse_filter_option():
    with patch(mock_print) as p:
        # tmp/a1/path1
        # tmp/a1 (1)/path1
        # tmp/a130/path1
        sort_and_print_entries(entries, Args(None, [(20, 100)], False, True, False))
        e = [call(entries[i]) for i in [2, 3, 4]]
        p.assert_has_calls(e)


def test_sort_and_print_entries_excludes_paths_23_or_130_with_exclude_option_list():
    with patch(mock_print) as p:
        # tmp/a1/path1
        # tmp/a1 (1)/path1
        # tmp/a57/path2
        # tmp/a64/path1
        # tmp/a64/path2
        sort_and_print_entries(entries, Args(None, None, [23, 130], True, False))
        e = [call(entries[i]) for i in [2, 3, 0, 5, 6]]
        p.assert_has_calls(e)


def test_sort_and_print_entries_reverses_order_with_reverse_option():
    with patch(mock_print) as p:
        # tmp/a130/path1
        # tmp/a64/path2
        # tmp/a64/path1
        # tmp/a57/path2
        # tmp/a23/path1
        # tmp/a1 (1)/path1
        # tmp/a1/path1
        sort_and_print_entries(entries, Args(None, None, False, True, True))
        e = [call(entries[i]) for i in reversed([2, 3, 1, 0, 5, 6, 4])]
        p.assert_has_calls(e)


# Each test has an "example" version for demonstrative purposes,
# and a test that uses the hypothesis module.

def test_range_check_returns_range_as_is_but_with_floats_if_first_is_less_than_second_example():
    assert range_check(10, 11) == (10.0, 11.0)
    assert range_check(6.4, 30) == (6.4, 30.0)


@given(x=integers(), data=data())  # Defer data selection for y till test is run.
def test_range_check_returns_range_as_is_but_with_floats_if_first_is_less_than_second(x, data):
    # Pull data such that the first is less than the second.
    y = data.draw(integers(min_value=x + 1))
    assert range_check(x, y) == (x, y)


@given(x=floats(allow_nan=False, min_value=-1E8, max_value=1E8), data=data())  # Defer data selection for y till test is run.
def test_range_check_returns_range_as_is_but_with_floats_if_first_is_less_than_second2(x, data):
    # Pull data such that the first is less than the second.
    y = data.draw(floats(min_value=x + 1.0, max_value=1E9, allow_nan=False))
    assert range_check(x, y) == (x, y)


def test_range_check_raises_ValueError_if_second_is_less_than_first_example():
    with raises(ValueError) as err:
        range_check(7, 2)
    assert str(err.value) == 'low >= high'


@given(x=floats(allow_nan=False), data=data())  # Defer data selection for y till test is run.
def test_range_check_raises_ValueError_if_second_is_less_than_first(x, data):
    # Pull data such that the first is greater than or equal to the second.
    y = data.draw(floats(max_value=x, allow_nan=False))
    with raises(ValueError) as err:
        range_check(x, y)
    assert str(err.value) == 'low >= high'


def test_check_filter_returns_None_if_filter_evaluates_to_False():
    assert check_filter(()) is None
    assert check_filter(False) is None
    assert check_filter(None) is None


def test_check_filter_returns_input_as_is_if_filter_is_valid_example():
    assert check_filter([(6, 7)]) == [(6, 7)]
    assert check_filter([(6, 7), (2, 8)]) == [(6, 7), (2, 8)]


@given(x=lists(integers(), min_size=1), data=data())  # Defer data selection for y till test is run.
def test_check_filter_returns_input_as_is_if_filter_is_valid(x, data):
    y = [data.draw(integers(min_value=val + 1)) for val in x]  # ensure y is element-wise greater than x
    assert check_filter(list(zip(x, y))) == [(i, j) for i, j in zip(x, y)]


def test_check_filter_raises_ValueError_if_filter_is_invalid_example():
    with raises(ValueError) as err:
        check_filter([(7, 2)])
    assert str(err.value) == 'Error in --filter: low >= high'


@given(x=lists(integers(), min_size=1), data=data())  # Defer data selection for y till test is run.
def test_check_filter_raises_ValueError_if_filter_is_invalid(x, data):
    y = [data.draw(integers(max_value=val)) for val in x]    # ensure y is element-wise less than or equal to x
    with raises(ValueError) as err:
        check_filter(list(zip(x, y)))
    assert str(err.value) == 'Error in --filter: low >= high'


def test_keep_entry_range_returns_True_if_any_portion_of_input_is_between_the_range_bounds_example():
    assert keep_entry_range('a56b23c89', [0], [100], int, re.compile(r'\d+'))


def test_keep_entry_range_returns_True_if_any_portion_of_input_is_between_any_range_bounds_example():
    assert keep_entry_range('a56b23c89', [1, 88], [20, 90], int, re.compile(r'\d+'))


def test_keep_entry_range_returns_False_if_no_portion_of_input_is_between_the_range_bounds_example():
    assert not keep_entry_range('a56b23c89', [1], [20], int, re.compile(r'\d+'))


def test_exclude_entry_returns_True_if_exlcude_parameters_are_not_in_input_example():
    assert exclude_entry('a56b23c89', [100, 45], int, re.compile(r'\d+'))


def test_exclude_entry_returns_False_if_exlcude_parameters_are_in_input_example():
    assert not exclude_entry('a56b23c89', [23], int, re.compile(r'\d+'))



# -*- coding: utf-8 -*-
"""Alternate versions of the splitting functions for testing."""
from __future__ import unicode_literals

import unicodedata
import collections
import itertools
import functools
from natsort.unicode_numbers import decimals
from natsort.compat.py23 import PY_VERSION, py23_zip

if PY_VERSION >= 3.0:
    long = int

triple_none = None, None, None
_sentinel = object()
SplitElement = collections.namedtuple('SplitElement',
                                      ['isnum', 'val', 'isuni'])


def int_splitter(iterable, signed, sep):
    """Alternate (slow) method to split a string into numbers."""
    iterable = unicodedata.normalize('NFD', iterable)
    split_by_decimal = itertools.groupby(iterable, lambda a: a.isdigit())
    split_by_decimal = refine_split_grouping(split_by_decimal)
    split = int_splitter_iter(split_by_decimal, signed)
    split = sep_inserter(split, sep)
    return tuple(add_leading_space_if_first_is_num(split, sep))


def float_splitter(iterable, signed, exp, sep):
    """Alternate (slow) method to split a string into numbers."""

    def number_tester(x):
        return x.isdecimal() or unicodedata.numeric(x, None) is not None

    iterable = unicodedata.normalize('NFD', iterable)
    split_by_decimal = itertools.groupby(iterable, number_tester)
    split_by_decimal = peekable(refine_split_grouping(split_by_decimal))
    split = float_splitter_iter(split_by_decimal, signed, exp)
    split = sep_inserter(split, sep)
    return tuple(add_leading_space_if_first_is_num(split, sep))


def refine_split_grouping(iterable):
    """Combines lists into strings, and separates unicode numbers from ASCII"""
    for isnum, values in iterable:
        values = list(values)
        # Further refine numbers into unicode and ASCII numeric characters.
        if isnum:
            num_grouped = group_unicode_and_ascii_numbers(values)
            for isuni, num_values in num_grouped:
                # If unicode, return one character at a time.
                if isuni:
                    for u in num_values:
                        yield SplitElement(True, u, True)
                # If ASCII, combine into a single multicharacter number.
                else:
                    val = ''.join(num_values)
                    yield SplitElement(True, val, False)

        else:
            # If non-numeric, combine into a single string.
            val = ''.join(values)
            yield SplitElement(False, val, False)


def group_unicode_and_ascii_numbers(
        iterable, ascii_digits=frozenset(decimals + '0123456789')
):
    """
    Use groupby to group ASCII and unicode numeric characters.
    Assumes all input is already all numeric characters.
    """
    return itertools.groupby(iterable, lambda a: a not in ascii_digits)


def int_splitter_iter(iterable, signed):
    """Split the input into integers and strings."""
    for isnum, val, isuni in iterable:
        if isuni:
            yield unicodedata.digit(val)
        elif isnum:
            yield int(val)
        elif signed:
            for x in try_to_read_signed_integer(iterable, val):
                yield int(''.join(x)) if isinstance(x, list) else x
        else:
            yield val


def float_splitter_iter(iterable, signed, exp):
    """Split the input into integers and other."""
    weird_check = ('-inf', '-infinity', '+inf', '+infinity',
                   'inf', 'infinity', 'nan', '-nan', '+nan')
    try_to_read_float_correctly = [
        try_to_read_float,
        try_to_read_float_with_exp,
        functools.partial(try_to_read_signed_float_template,
                          key=try_to_read_float),
        functools.partial(try_to_read_signed_float_template,
                          key=try_to_read_float_with_exp),
    ][signed * 2 + exp * 1]  # Choose the appropriate converter function.
    for isnum, val, isuni in iterable:
        if isuni:
            yield unicodedata.numeric(val)
        else:
            for x in try_to_read_float_correctly(iterable, isnum, val):
                if isinstance(x, list):
                    yield float(''.join(x))
                elif x.lower().strip(' \t\n\r\f\v') in weird_check:
                    yield float(x)
                else:
                    yield x


def try_to_read_signed_integer(iterable, val):
    """
    If the given string ends with +/-, attempt to return a signed int.
    Otherwise, return the string as-is.
    """
    if val.endswith(('+', '-')):
        next_element = next(iterable, None)

        # Last element, return as-is.
        if next_element is None:
            yield val
            return

        # We know the next value in the sequence must be "isnum == True".
        # We just need to handle unicode or not.
        _, next_val, next_isuni = next_element

        # If unicode, don't apply sign and just return the val as-is
        # and convert the unicode character.
        if next_isuni:
            yield val
            yield unicodedata.digit(next_val)

        # If the val is *only* the sign, return only the number.
        elif val in ('-', '+'):
            yield [val, next_val]

        # Otherwise, remove the sign from the val and apply it to the number,
        # returning both.
        else:
            yield val[:-1]
            yield [val[-1], next_val]

    else:
        yield val


def try_to_read_float(iterable, isnum, val):
    """
    Try to read a string that matches num.num and return as a float.
    Otherwise return the input as found.
    """
    # Extract what is coming next.
    next_isnum, next_val, next_isuni = iterable.peek(triple_none)

    # If a non-number was given, we can only accept a decimal point.
    if not isnum:

        # If the next value is None or not a non-uni number, return as-is.
        if next_val is None or not next_isnum or next_isuni:
            yield val

        # If this the decimal point, add it to the number and return.
        elif val == '.':
            next(iterable)  # To progress the iterator.
            yield [val, next_val]

        # If the val ends with the decimal point, split the decimal point
        # off the end of the string then place it to the front of the
        # iterable so that we can use it later.
        elif val.endswith('.'):
            iterable.push(SplitElement(False, val[-1], False))
            yield val[:-1]

        # Otherwise, just return the val and move on.
        else:
            yield val

    # If a number, read the number then try to get the post-decimal part.
    else:

        # If the next element is not '.', return now.
        if next_val != '.':
            # If the next val starts with a '.', let's add that.
            if next_val is not None and next_val.startswith('.'):
                next(iterable)  # To progress the iterator.
                iterable.push(SplitElement(False, next_val[1:], False))
                yield [val, next_val[0]]
            else:
                yield [val]

        # Recursively parse the decimal and after. If the returned
        # value is a list, add the list to the current number.
        # If not, just return the number with the decimal.
        else:
            # If the first value returned from the try_to_read_float
            # is a list, add it to the float component list.
            next(iterable)  # To progress the iterator.
            ret = next(try_to_read_float(iterable, next_isnum, next_val))
            if isinstance(ret, list):
                yield [val] + ret
            else:
                yield [val, next_val]


def try_to_read_float_with_exp(iterable, isnum, val):
    """
    Try to read a string that matches num.numE[+-]num and return as a float.
    Otherwise return the input as found.
    """
    exp_ident = ('e', 'E', 'e-', 'E-', 'e+', 'E+')

    # Start by reading the floating point part.
    float_ret = next(try_to_read_float(iterable, isnum, val))

    # Extract what is coming next.
    next_isnum, next_val, next_isuni = iterable.peek(triple_none)

    # If the float part is not a list, or the next value
    # is not in the exponential identifier list, return it as-is.
    if not isinstance(float_ret, list) or next_val not in exp_ident:
        yield float_ret

    # We know the next_val is an exponential identifier. See if the value
    # after that is a non-unicode number. If so, return all as a float.
    # If not, put the exponential identifier back on the front of the
    # list and return the float_ret as-is.
    else:
        exp = SplitElement(next_isnum, next_val, next_isuni)
        next(iterable)  # To progress the iterator.
        next_isnum, next_val, next_isuni = iterable.peek(triple_none)
        if next_isnum and not next_isuni:
            next(iterable)  # To progress the iterator.
            yield float_ret + [exp.val, next_val]
        else:
            iterable.push(exp)
            yield float_ret


def try_to_read_signed_float_template(iterable, isnum, val, key):
    """
    Try to read a string that matches [+-]num.numE[+-]num and return as a
    float. Otherwise return the input as found.
    """
    # Extract what is coming next.
    next_isnum, next_val, next_isuni = iterable.peek(triple_none)

    # If it looks like there is a sign here and the next value is a
    # non-unicode number, try to parse that with the sign.
    if val.endswith(('+', '-')) and next_isnum and not next_isuni:

        # If this value is a sign, return the combo.
        if val in ('+', '-'):
            next(iterable)  # To progress the iterator.
            yield [val] + next(key(iterable, next_isnum, next_val))

        # If the val ends with the sign split the sign off the end of
        # the string then place it to the front of the iterable so that
        # we can use it later.
        else:
            iterable.push(SplitElement(False, val[-1], False))
            yield val[:-1]

    # If it looks like there is a sign here and the next value is a
    # decimal, try to parse as a decimal.
    elif val.endswith(('+.', '-.')) and next_isnum and not next_isuni:

        # Push back a zero before the decimal then parse.
        print(val, iterable.peek())

        # If this value is a sign, return the combo
        if val[:-1] in ('+', '-'):
            yield [val[:-1]] + next(key(iterable, False, val[-1]))

        # If the val ends with the sign split the decimal the end of
        # the string then place it to the front of the iterable so that
        # we can use it later.
        else:
            iterable.push(SplitElement(False, val[-2:], False))
            yield val[:-2]

    # If no sign, pass directly to the key function.
    else:
        yield next(key(iterable, isnum, val))


def add_leading_space_if_first_is_num(iterable, sep):
    """Check if the first element is a number, and prepend with space if so."""
    z, peek = itertools.tee(iterable)
    if type(next(peek, None)) in (int, long, float):
        z = itertools.chain([sep], z)
    del peek
    return z


def sep_inserter(iterable, sep, types=frozenset((int, long, float))):
    """Simulates the py3_safe function."""
    pairs = pairwise(iterable)

    # Prime loop by handling first pair specially.
    try:
        first, second = next(pairs)
    except StopIteration:
        return
    if second is None:  # Only one element
        yield first
    elif type(first) in types and type(second) in types:
        yield first
        yield sep
        yield second
    else:
        yield first
        yield second

    # Handle all remaining pairs in loop.
    for first, second in pairs:
        if type(first) in types and type(second) in types:
            yield sep
        yield second


def pairwise(iterable):
    "s -> (s0,s1), (s1,s2), (s2,s3), ..."
    split1, split2 = itertools.tee(iterable)
    a, b = itertools.tee(split1)
    test1, test2 = itertools.tee(split2)
    next(b, None)
    if next(test1, None) is None:
        ret = py23_zip(a, b)  # Returns empty list
    elif next(test2, None) is not None and next(test2, None) is None:
        ret = py23_zip(a, [None])  # Return at least one value
    else:
        ret = py23_zip(a, b)
    del test1, test2, split2
    return ret


class peekable(object):
    """Wrapper for an iterator to allow 1-item lookahead
    Call ``peek()`` on the result to get the value that will next pop out of
    ``next()``, without advancing the iterator:
        >>> p = peekable(xrange(2))
        >>> p.peek()
        0
        >>> p.next()
        0
        >>> p.peek()
        1
        >>> p.next()
        1
    Pass ``peek()`` a default value, and it will be returned in the case where
    the iterator is exhausted:
        >>> p = peekable([])
        >>> p.peek('hi')
        'hi'
    If no default is provided, ``peek()`` raises ``StopIteration`` when there
    are no items left.
    To test whether there are more items in the iterator, examine the
    peekable's truth value. If it is truthy, there are more items.
        >>> assert peekable(xrange(1))
        >>> assert not peekable([])
    """
    # Lowercase to blend in with itertools. The fact that it's a class is an
    # implementation detail.

    def __init__(self, iterable):
        self._it = iter(iterable)

    def __iter__(self):
        return self

    def __nonzero__(self):
        try:
            self.peek()
        except StopIteration:
            return False
        return True

    __bool__ = __nonzero__

    def peek(self, default=_sentinel):
        """Return the item that will be next returned from ``next()``.
        Return ``default`` if there are no items left. If ``default`` is not
        provided, raise ``StopIteration``.
        """
        if not hasattr(self, '_peek'):
            try:
                self._peek = next(self._it)
            except StopIteration:
                if default is _sentinel:
                    raise
                return default
        return self._peek

    def next(self):
        ret = self.peek()
        try:
            del self._peek
        except AttributeError:
            pass
        return ret

    __next__ = next

    def push(self, value):
        """Put an element at the front of the iterable."""
        if hasattr(self, '_peek'):
            self._it = itertools.chain([value, self._peek], self._it)
            del self._peek
        else:
            self._it = itertools.chain([value], self._it)



# -*- coding: utf-8 -*-
"""This module defines the "ns" enum for natsort."""
from __future__ import (
    print_function,
    division,
    unicode_literals,
    absolute_import
)


class ns(object):
    """
    Enum to control the `natsort` algorithm.

    This class acts like an enum to control the `natsort` algorithm. The
    user may select several options simultaneously by or'ing the options
    together.  For example, to choose ``ns.INT``, ``ns.PATH``, and
    ``ns.LOCALE``, you could do ``ns.INT | ns.LOCALE | ns.PATH``. Each
    function in the :mod:`natsort` package has an `alg` option that accepts
    this enum to allow fine control over how your input is sorted.

    Each option has a shortened 1- or 2-letter form.

    .. note:: Please read :ref:`locale_issues` before using ``ns.LOCALE``.

    Attributes
    ----------
    INT, I (default)
        The default - parse numbers as integers.
    FLOAT, F
        Tell `natsort` to parse numbers as floats.
    UNSIGNED, U (default)
        Tell `natsort` to ignore any sign (i.e. "-" or "+") to the immediate
        left of a number.  This is the default.
    SIGNED, S
        Tell `natsort` to take into account any sign (i.e. "-" or "+")
        to the immediate left of a number.
    REAL, R
        This is a shortcut for ``ns.FLOAT | ns.SIGNED``, which is useful
        when attempting to sort real numbers.
    NOEXP, N
        Tell `natsort` to not search for exponents as part of a float number.
        For example, with `NOEXP` the number "5.6E5" would be interpreted
        as `5.6`, `"E"`, and `5` instead of `560000`.
    NUMAFTER, NA
        Tell `natsort` to sort numbers after non-numbers. By default
        numbers will be ordered before non-numbers.
    PATH, P
        Tell `natsort` to interpret strings as filesystem paths, so they
        will be split according to the filesystem separator
        (i.e. '/' on UNIX, '\\' on Windows), as well as splitting on the
        file extension, if any. Without this, lists of file paths like
        ``['Folder/', 'Folder (1)/', 'Folder (10)/']`` will not be
        sorted properly; 'Folder/' will be placed at the end, not at the
        front. It is the same as setting the old `as_path` option to
        `True`.
    COMPATIBILITYNORMALIZE, CN
        Use the "NFKD" unicode normalization form on input rather than the
        default "NFD". This will transform characters such as '⑦' into
        '7'. Please see https://stackoverflow.com/a/7934397/1399279,
        https://stackoverflow.com/a/7931547/1399279,
        and http://unicode.org/reports/tr15/ for full details into unicode
        normalization.
    LOCALE, L
        Tell `natsort` to be locale-aware when sorting. This includes both
        proper sorting of alphabetical characters as well as proper
        handling of locale-dependent decimal separators and thousands
        separators. This is a shortcut for
        ``ns.LOCALEALPHA | ns.LOCALENUM``.
        Your sorting results will vary depending on your current locale.
    LOCALEALPHA, LA
        Tell `natsort` to be locale-aware when sorting, but only for
        alphabetical characters.
    LOCALENUM, LN
        Tell `natsort` to be locale-aware when sorting, but only for
        decimal separators and thousands separators.
    IGNORECASE, IC
        Tell `natsort` to ignore case when sorting.  For example,
        ``['Banana', 'apple', 'banana', 'Apple']`` would be sorted as
        ``['apple', 'Apple', 'Banana', 'banana']``.
    LOWERCASEFIRST, LF
        Tell `natsort` to put lowercase letters before uppercase letters
        when sorting.  For example,
        ``['Banana', 'apple', 'banana', 'Apple']`` would be sorted as
        ``['apple', 'banana', 'Apple', 'Banana']`` (the default order
        would be ``['Apple', 'Banana', 'apple', 'banana']`` which is
        the order from a purely ordinal sort).
        Useless when used with `IGNORECASE`. Please note that if used
        with ``LOCALE``, this actually has the reverse effect and will
        put uppercase first (this is because ``LOCALE`` already puts
        lowercase first); you may use this to your advantage if you
        need to modify the order returned with ``LOCALE``.
    GROUPLETTERS, G
        Tell `natsort` to group lowercase and uppercase letters together
        when sorting.  For example,
        ``['Banana', 'apple', 'banana', 'Apple']`` would be sorted as
        ``['Apple', 'apple', 'Banana', 'banana']``.
        Useless when used with `IGNORECASE`; use with `LOWERCASEFIRST`
        to reverse the order of upper and lower case. Generally not
        needed with `LOCALE`.
    CAPITALFIRST, C
        Only used when `LOCALE` is enabled. Tell `natsort` to put all
        capitalized words before non-capitalized words. This is essentially
        the inverse of `GROUPLETTERS`, and is the default Python sorting
        behavior without `LOCALE`.
    UNGROUPLETTERS, UG
        An alias for `CAPITALFIRST`.
    NANLAST, NL
        If an NaN shows up in the input, this instructs `natsort` to
        treat these as +Infinity and place them after all the other numbers.
        By default, an NaN be treated as -Infinity and be placed first.
    TYPESAFE, T
        Deprecated as of `natsort` version 5.0.0; this option is now
        a no-op because it is always true.
    VERSION, V
        Deprecated as of `natsort` version 5.0.0; this option is now
        a no-op because it is the default.
    DIGIT, D
        Same as `VERSION` above.

    Notes
    -----
    If you prefer to use `import natsort as ns` as opposed to
    `from natsort import natsorted, ns`, the `ns` options are
    available as top-level imports.

        >>> import natsort as ns
        >>> a = ['num5.10', 'num-3', 'num5.3', 'num2']
        >>> ns.natsorted(a, alg=ns.REAL) == ns.natsorted(a, alg=ns.ns.REAL)
        True

    """
    # Following were previously now options but are now defaults.
    TYPESAFE         = T  = 0
    INT              = I  = 0
    VERSION          = V  = 0
    DIGIT            = D  = 0
    UNSIGNED         = U  = 0

    # The below are options. The values are stored as powers of two
    # so bitmasks can be used to extract the user's requested options.
    FLOAT                  = F  = 1 << 0
    SIGNED                 = S  = 1 << 1
    REAL                   = R  = FLOAT | SIGNED
    NOEXP                  = N  = 1 << 2
    PATH                   = P  = 1 << 3
    LOCALEALPHA            = LA = 1 << 4
    LOCALENUM              = LN = 1 << 5
    LOCALE                 = L  = LOCALEALPHA | LOCALENUM
    IGNORECASE             = IC = 1 << 6
    LOWERCASEFIRST         = LF = 1 << 7
    GROUPLETTERS           = G  = 1 << 8
    UNGROUPLETTERS         = UG = 1 << 9
    CAPITALFIRST           = C  = UNGROUPLETTERS
    NANLAST                = NL = 1 << 10
    COMPATIBILITYNORMALIZE = CN = 1 << 11
    NUMAFTER               = NA = 1 << 12

    # The below are private options for internal use only.
    _NUMERIC_ONLY    = REAL | NOEXP
    _DUMB            = 1 << 31
